import numpy as np
import re
import logging
import matplotlib.pyplot as plt
import sys


class FermiSurface:
    def __init__(self, kpoints, bands, spd, loglevel=logging.WARNING):
        """FermiSurface: Class to build and to plot a 2D fermi surface.  It
    finds the relevant bands (crossig the Fermi level) and interpolate
    them
    
    args:
      kpoints: Numpy array with kpoints Nx3.
      bands: the bands with Fermi energy already substracted!, numpy array,
             Nkpoints x Nbands.
      spd: character (atomic, orbital) of each bands at each Kpoint, numpy
           array Nkpoints x Nbands.
      loglevel(=logging.WARNING): the verbosity level. 
    """
        # Since some time ago Kpoints are in cartesian coords (ready to use)
        self.kpoints = kpoints
        self.bands = bands
        self.spd = spd
        self.useful = None  # List of useful bands (filled in findEnergy)
        self.energy = None

        self.log = logging.getLogger("FermiSurface")
        self.log.setLevel(loglevel)
        self.ch = logging.StreamHandler()
        self.ch.setFormatter(
            logging.Formatter("%(name)s::%(levelname)s: " "%(message)s")
        )
        self.ch.setLevel(logging.DEBUG)
        self.log.addHandler(self.ch)

        self.log.debug("FermiSurface.init: ...")
        self.log.info("Kpoints.shape : " + str(self.kpoints.shape))
        self.log.info("bands.shape   : " + str(self.bands.shape))
        self.log.info("spd.shape     : " + str(self.spd.shape))
        self.log.debug("FermiSurface.init: ...Done")
        return

    def FindEnergy(self, energy):
        self.log.debug("FindEnergy: ...")
        self.energy = energy
        self.log.info("Energy   : " + str(energy))
        bands = self.bands.transpose()
        # searching for bands crossing the desired energy
        self.useful = np.where(
            np.logical_and(bands.min(axis=1) < energy, bands.max(axis=1) > energy)
        )
        self.log.info("set of useful bands    : " + str(self.useful))
        bands = bands[self.useful]
        self.log.debug("new bands.shape : " + str(bands.shape))
        if len(bands) == 0:
            self.log.error("No bands found in that range. Check your data. Returning")
            raise RuntimeError("No bands to plot")
        self.log.debug("FindEnergy: ...Done")
        return

    def Plot(self, interpolation=200, mask=None):
        """Only 2D layer geometry along z"""
        self.log.debug("Plot: ...")
        from scipy.interpolate import griddata

        if self.useful is None:
            raise RuntimeError("self.FindEnergy() must be called before Plotting")

        # selecting components of K-points
        x, y = self.kpoints[:, 0], self.kpoints[:, 1]
        self.log.debug("k_x[:10], k_y[:10] values" + str([x[:10], y[:10]]))

        bands = self.bands.transpose()[self.useful]

        # and new, interpolated component
        xmax, xmin = x.max(), x.min()
        ymax, ymin = y.max(), y.min()
        self.log.debug("xlim = " + str([xmin, xmax]) + "  ylim = " + str([ymin, ymax]))
        xnew, ynew = np.mgrid[
            xmin : xmax : interpolation * 1j, ymin : ymax : interpolation * 1j
        ]

        # interpolation
        bnew = []
        for band in bands:
            self.log.debug("Interpolating ...")
            bnew.append(griddata((x, y), band, (xnew, ynew), method="cubic"))

        plots = [
            plt.contour(
                xnew,
                ynew,
                z,
                [self.energy],
                linewidths=0.5,
                colors="k",
                linestyles="solid",
            )
            for z in bnew
        ]
        plt.axis("equal")
        self.log.debug("Plot: ...Done")
        return plots

    def st(self, sx, sy, sz, spin=None, noarrow=False, interpolation=300):
        """Only 2D layer geometry along z. It is like a enhanced version
    of 'plot' method.

    sx, sy, sz are spin projected Nkpoints x Nbands numpy arrays. They
    also are (already) projected by orbital and atom (from other
    class)

    """
        self.log.debug("st: ...")
        from scipy.interpolate import griddata

        if self.useful is None:
            raise RuntimeError("self.FindEnergy() must be called before Plotting")

        # selecting components of K-points
        x, y = self.kpoints[:, 0], self.kpoints[:, 1]

        bands = self.bands.transpose()[self.useful]

        sx = sx.transpose()[self.useful]
        sy = sy.transpose()[self.useful]
        sz = sz.transpose()[self.useful]

        # and new, interpolated component
        xmax, xmin = x.max(), x.min()
        ymax, ymin = y.max(), y.min()
        self.log.debug("xlim = " + str([xmin, xmax]) + "  ylim = " + str([ymin, ymax]))
        xnew, ynew = np.mgrid[
            xmin : xmax : interpolation * 1j, ymin : ymax : interpolation * 1j
        ]

        # interpolation
        bnew = []
        for band in bands:
            self.log.debug("Interpolating ...")
            bnew.append(griddata((x, y), band, (xnew, ynew), method="cubic"))

        linewidths = 0.7
        if noarrow:
            # print "second no arrow\n"
            linewidths = 0.2
        cont = [
            plt.contour(
                xnew,
                ynew,
                z,
                [self.energy],
                linewidths=linewidths,
                colors="k",
                linestyles="solid",
            )
            for z in bnew
        ]
        plt.axis("equal")

        for (contour, spinX, spinY, spinZ) in zip(cont, sx, sy, sz):
            # The previous interp. yields the level curves, nothing more is
            # useful from there
            paths = contour.collections[0].get_paths()
            verts = [xx.vertices for xx in paths]
            points = np.concatenate(verts)
            self.log.debug("Fermi surf. points.shape: " + str(points.shape))

            newSx = griddata((x, y), spinX, (points[:, 0], points[:, 1]))
            newSy = griddata((x, y), spinY, (points[:, 0], points[:, 1]))
            newSz = griddata((x, y), spinZ, (points[:, 0], points[:, 1]))

            self.log.info("newSx.shape: " + str(newSx.shape))

            import matplotlib.colors as colors

            if noarrow is False:
                plt.quiver(
                    points[::6, 0],
                    points[::6, 1],
                    newSx[::6],
                    newSy[::6],
                    newSz[::6],
                    scale_units="xy",
                    angles="xy",
                    norm=colors.Normalize(-0.5, 0.5),
                )
            else:
                # a dictionary to select the right spin component
                spinDict = {1: newSx[::6], 2: newSy[::6], 3: newSz[::6]}
                plt.scatter(
                    points[::6, 0],
                    points[::6, 1],
                    c=spinDict[spin],
                    s=50,
                    edgecolor="none",
                    alpha=1.0,
                    marker=".",
                    cmap="seismic",
                    norm=colors.Normalize(-0.5, 0.5),
                )
        plt.colorbar()
        plt.axis("equal")
        font = {"size": 16}
        plt.rc("font", **font)

        self.log.debug("st: ...Done")
        return



# -*- coding: utf-8 -*-

import numpy as np
import pyvista
from matplotlib import colors as mpcolors
from matplotlib import cm
from .fermisurface3d import boolean_add
from .fermisurface3d import FermiSurface3D
from .splash import welcome
from .fermisurface3d import FermiSurface3D
from .utilsprocar import UtilsProcar
from .procarparser import ProcarParser
from .procarselect import ProcarSelect

def fermi3D(procar='PROCAR',
            outcar='OUTCAR',
            fermi=None,
            bands=None, 
            interpolation_factor=1,
            mode="plain",
            supercell=[1,1,1],
            colors=None,
            background_color='white',
            save_colors=None,
            cmap='viridis',
            atoms=None,
            orbitals=None,
            spin=None,
            spin_texture=False,
            arrow_color=None,
            arrow_size=0.015,
            only_spin=False,
            fermi_shift=0,
            projection_accuracy='normal',
            code='vasp',
            vmin=0,
            vmax=1,
            savegif=None,
            savemp4=None,
            save3d=None,
            perspective=True,
            save2d=False,
            camera_pos=[1,1,1],
            widget=None,
            show=True,
            ):
    
    
    
    
    welcome()
    
    
    if show:
        p = pyvista.Plotter()
    
    if code=='vasp':
        outcarparser = UtilsProcar()
        if fermi is None:
            e_fermi = outcarparser.FermiOutcar(outcar)
        else:
            e_fermi = fermi
        reciprocal_lattice = outcarparser.RecLatOutcar(outcar)
        procarFile = ProcarParser()
        procarFile.readFile(procar, False)
        data = ProcarSelect(procarFile, deepCopy=True)
    if bands is None:
        bands = np.arange(len(data.bands[0,:]))
    surfaces = []
    
        
        
    spd = []        
    if mode == 'parametric':
        if orbitals is None:
            orbitals = [-1]
        if atoms is None:
            atoms = [-1]
        if spin is None:
            spin = [0]
        data.selectIspin(spin)
        data.selectAtoms(atoms, fortran=False)
        data.selectOrbital(orbitals)

        for iband in bands:
            spd.append(data.spd[:,iband])
    else :
        for iband in bands:
            spd.append(None)

    spd_spin=[]
    
    if spin_texture:
        dataX = ProcarSelect(procarFile, deepCopy=True)
        dataY = ProcarSelect(procarFile, deepCopy=True)
        dataZ = ProcarSelect(procarFile, deepCopy=True)

        dataX.kpoints = data.kpoints
        dataY.kpoints = data.kpoints
        dataZ.kpoints = data.kpoints

        dataX.spd = data.spd
        dataY.spd = data.spd
        dataZ.spd = data.spd

        dataX.bands = data.bands
        dataY.bands = data.bands
        dataZ.bands = data.bands

        dataX.selectIspin([1])
        dataY.selectIspin([2])
        dataZ.selectIspin([3])

        dataX.selectAtoms(atoms, fortran=False)
        dataY.selectAtoms(atoms, fortran=False)
        dataZ.selectAtoms(atoms, fortran=False)

        dataX.selectOrbital(orbitals)
        dataY.selectOrbital(orbitals)
        dataZ.selectOrbital(orbitals)
        for iband in bands:
            spd_spin.append([dataX.spd[:,iband],dataY.spd[:,iband],dataZ.spd[:,iband]])
    else:
        for iband in bands:
            spd_spin.append(None)
    counter = 0
    for iband in bands:
        print("Trying to extract isosurface for band %d"%iband)
        surface = FermiSurface3D(kpoints=data.kpoints,
                                 band=data.bands[:,iband],
                                 spd=spd[counter],
                                 spd_spin=spd_spin[counter],
                                 fermi=e_fermi+fermi_shift,
                                 reciprocal_lattice=reciprocal_lattice,
                                 interpolation_factor=interpolation_factor,
                                 projection_accuracy=projection_accuracy,
                                 supercell=supercell)
        if surface.verts is not None:
            surfaces.append(surface)       
        counter+=1

        
        
    nsurface = len(surfaces)
    norm = mpcolors.Normalize(vmin=vmin, vmax=vmax)
    cmap = cm.get_cmap(cmap)
    scalars = np.arange(nsurface+1)/nsurface
    if colors is None:
        colors = np.array([cmap(norm(x)) for x in (scalars)]).reshape(-1,4)
    
    
    if show or save2d:
        # sargs = dict(interactive=True) 
        p.add_mesh(surfaces[0].brillouin_zone.pyvista_obj,
                    style='wireframe',line_width=3.5,color='black')
        for isurface in range(nsurface):
            if not only_spin:
                if mode == 'plain':
                    p.add_mesh(surfaces[isurface].pyvista_obj,
                               color=colors[isurface])
                    text='Colors'
                elif mode =='parametric':
                    p.add_mesh(surfaces[isurface].pyvista_obj,
                               cmap=cmap,
                               clim=[vmin,vmax])
                    p.remove_scalar_bar()
                    text='Projection'
            else:
                text='Spin Texture'
            if spin_texture:
                # Example dataset with normals
                # create a subset of arrows using the glyph filter
                arrows = surfaces[isurface].pyvista_obj.glyph(orient="vectors",factor=arrow_size)

                if arrow_color is None:
                    p.add_mesh(arrows,
                               cmap=cmap,
                               clim=[vmin,vmax])
                    p.remove_scalar_bar()
                else:
                    p.add_mesh(arrows,
                               color=arrow_color)

        p.add_scalar_bar(title=text,
                         n_labels=6,
                         italic=False,
                         bold=False,
                         title_font_size=None,
                         label_font_size=None,
                         position_x=0.9,
                         position_y=0.01,
                         color='black')
                
        p.add_axes(xlabel='Kx',
                   ylabel='Ky',
                   zlabel='Kz',
                   line_width=6,
                   labels_off=False)

        if not perspective :
            p.enable_parallel_projection()
        p.set_background(background_color)
        p.set_position(camera_pos)
        p.show(cpos=camera_pos,screenshot=save2d)
        # p.screenshot('1.png')
        # p.save_graphic('1.pdf')
        if savegif is not None:
            path = p.generate_orbital_path(n_points=36)
            p.open_gif(savegif)
            p.orbit_on_path(path)#,viewup=camera_pos)
        if savemp4:
            path = p.generate_orbital_path(n_points=36)
            p.open_movie(savemp4)
            p.orbit_on_path(path)#,viewup=camera_pos)
            # p.close()        
    if save_colors is None:
        for i in range(nsurface):
            if surfaces[i].scalars is None:
                surfaces[i].set_scalars([scalars[i]]*surfaces[i].nfaces)
    s = boolean_add(surfaces)
    s.set_color_with_cmap(cmap=cmap,vmin=vmin,vmax=vmax)
    # s.pyvista_obj.plot()
    # s.trimesh_obj.show()
    
    if save3d is not None:
        extention=save3d.split('.')[-1]
        s.export(save3d,extention)
    return s,surfaces



#!/usr/bin/env python

"""
Stand alone script for PyProcar.

This calls the modules in the /pyprocar directory.

Based on the original script developed by Aldo Romero (alromero@mail.wvu.edu) and
Francisco Munoz (fvmunoz@gmail.com).
"""

import argparse
from argparse import RawTextHelpFormatter
import pyprocar
import numpy as np


def call_bandsplot(args):
    """
	This module calls the band structure plotting function.
	"""

    pyprocar.bandsplot(
        args.file,
        mode=args.mode,
        color=args.color,
        abinit_output=args.abinit,
        spin=args.spin,
        atoms=args.atoms,
        orbitals=args.orbitals,
        fermi=args.fermi,
        elimit=args.elimit,
        mask=args.mask,
        markersize=args.markersize,
        cmap=args.cmap,
        vmax=args.vmax,
        vmin=args.vmin,
        grid=args.grid,
        marker=args.marker,
        permissive=args.permissive,
        human=args.human,
        savefig=args.savefig,
        kticks=args.kticks,
        knames=args.knames,
        title=args.title,
        outcar=args.outcar,
        kpointsfile=args.kpointsfile,
        kdirect=args.kdirect,
    )


def call_kpath(args):
    """
	This module calls the k-path generation function.
	"""

    pyprocar.kpath(
        args.infile,
        grid_size=args.grid_size,
        with_time_reversal=args.with_time_reversal,
        recipe=args.recipe,
        threshold=args.threshold,
        symprec=args.symprec,
        angle_tolerence=args.angle_tolerence,
        supercell_matrix=args.supercell_matrix,
    )


def call_repair(args):
    """
	This module calls the repair function.
	"""
    pyprocar.repair(args.infile, args.outfile)


def call_generate2dkmesh(args):
    """
	This module calls the k-mesh generating function.
	"""
    pyprocar.generate2dkmesh(
        args.x1, args.y1, args.x2, args.y2, args.z, args.nkx, args.nky
    )


def call_fermi2D(args):
    """
	This module calls the fermi2D plotting function.
	"""
    pyprocar.fermi2D(
        args.file,
        outcar=args.outcar,
        spin=args.spin,
        atoms=args.atoms,
        orbitals=args.orbitals,
        energy=args.energy,
        fermi=args.fermi,
        rec_basis=args.rec_basis,
        rot_symm=args.rot_symm,
        translate=args.translate,
        rotation=args.rotation,
        human=args.human,
        mask=args.mask,
        savefig=args.savefig,
        st=args.st,
        noarrow=args.noarrow,
    )


def call_fermi3D(args):
    """
	This module calls the fermi3D plotting function.
	"""
    pyprocar.fermi3D(
        args.procar,
        args.outcar,
        args.bands,
        scale=args.scale,
        mode=args.mode,
        st=args.st,
        kwargs=args.kwargs,
    )


def call_filter(args):
    """
	This module calls the filter function.
	"""

    pyprocar.filter(
        args.infile,
        args.outfile,
        atoms=args.atoms,
        orbitals=args.orbitals,
        orbital_names=args.orbital_names,
        bands=args.bands,
        spin=args.spin,
        human_atoms=args.human_atoms,
    )


def call_cat(args):
    """
	This module calls the cat function.
	"""
    pyprocar.cat(inFiles=args.inFiles, outFile=args.outFile, gz=args.gz)


def call_mergeabinit(args):
    """
	This module calls the mergeabinit function.
	"""
    pyprocar.mergeabinit(args.outfile)


def call_bandgap(args):
    """
	This module calls the mergeabinit function.
	"""
    pyprocar.bandgap(args.procar, args.outcar, args.code, args.fermi)


def call_unfold(args):
    """
	This module calls the band unfolding function.
	"""
    pyprocar.unfold(
        fname=args.fname,
        poscar=args.poscar,
        outcar=args.outcar,
        supercell_matrix=args.supercell_matrix,
        ispin=args.ispin,
        efermi=args.efermi,
        shift_efermi=args.shift_efermi,
        elimit=args.elimit,
        kticks=args.kticks,
        knames=args.knames,
        print_kpts=args.print_kpts,
        show_band=args.show_band,
        width=args.width,
        color=args.color,
        savetab=args.savetab,
        savefig=args.savefig,
    )


if __name__ == "__main__":

    import sys

    args = sys.argv[1:]

    if args:

        # Top level parser
        description = "PyProcar: A Python library for analyzing PROCAR files."
        parser = argparse.ArgumentParser(description=description)
        subparsers = parser.add_subparsers(help="sub-command help")

        ############### cat ############################################

        phelp = (
            "concatenation of PROCARs files, they should be compatible (ie: "
            "joining parts of a large bandstructure calculation)."
        )
        parserCat = subparsers.add_parser("cat", help=phelp)

        phelp = "Input files. They can be compressed"
        parserCat.add_argument("inFiles", nargs="+", help=phelp)

        phelp = "Output file."
        parserCat.add_argument("outFile", help=phelp)

        phelp = "Writes a gzipped outfile (if needed a .gz extension automatically will be added)"
        parserCat.add_argument("--gz", help=phelp, action="store_true")

        parserCat.set_defaults(func=call_cat)

        ################ mergeabinit ##################################

        parsermergeabinit = subparsers.add_parser(
            "mergeabinit", help="Merge Abinit PROCARs."
        )
        parsermergeabinit.add_argument("outfile", help="Merged PROCAR")
        parsermergeabinit.set_defaults(func=call_mergeabinit)

        ############### unfold #######################################
        parserunfold = subparsers.add_parser("unfold", help="Band unfolding.")
        parserunfold.add_argument("-fname", help="PROCAR filename.")
        parserunfold.add_argument("-poscar", help="POSCAR filename.")
        parserunfold.add_argument(
            "-outcar", help="OUTCAR filename. Used to get Fermi energy."
        )
        parserunfold.add_argument(
            "-supercell_matrix",
            help="Supercell matrix from primitive cell to supercell.",
        )
        parserunfold.add_argument(
            "-ispin",
            help="None - non spin polarized \n 1 - spin up" "\n 2 - spin down.",
            choices=[None, 1, 2],
        )
        parserunfold.add_argument(
            "-efermi", help="Fermi energy. Only when no OUTCAR is given."
        )
        parserunfold.add_argument("-elimit", help="Range of energy to be plotted.")
        parserunfold.add_argument(
            "-kticks",
            help="The indices of k-points for labels.",
            type=int,
            nargs="+",
            action="append",
        )
        parserunfold.add_argument(
            "-knames", help="Labels of k-points.", type=str, nargs="+", action="append"
        )
        parserunfold.add_argument(
            "-print_kpts", help="Print all the k-points to screen.", action="store_true"
        )
        parserunfold.add_argument(
            "-show_band",
            help="Whether to plot the bands before unfolding.",
            action="store_true",
        )
        parserunfold.add_argument(
            "-width", help="Width of the unfolded band.", type=float
        )
        parserunfold.add_argument(
            "-color", help="Color of the unfolded band.", type=str
        )
        parserunfold.add_argument(
            "-savetab",
            help="The csv file name of which the table of unfolding result will be written into",
        )
        parserunfold.add_argument(
            "-savefig", help="The file name of which the figure will be saved."
        )
        parserunfold.set_defaults(func=call_unfold)

        ############### filter ##########################################
        phelp = (
            "Filters (manipulates) the data of the input file (PROCAR-like) and"
            " it yields a new file (PROCAR-like too) with the changes. This "
            "method can do only one manipulation at time (ie: spin, atoms, "
            "bands or orbitals)."
        )
        parserFilter = subparsers.add_parser("filter", help=phelp)

        phelp = "Input file. Can be compressed"
        parserFilter.add_argument("inFile", help=phelp)

        phelp = "Output file."
        parserFilter.add_argument("outFile", help=phelp)

        OptFilter = parserFilter.add_mutually_exclusive_group()
        phelp = (
            "List of atoms to group (add) as a new single entry. Each group of"
            " atoms should be specified in a different `--atoms` option. "
            "Example: `procar.py filter in out -a 0 1 -a 2` will group the 1st"
            " and 2nd atoms, while keeping the 3rd atom in `out` (any atom "
            "beyond the 3rd will be discarded). Mind the last atomic field "
            "present on a PROCAR file, is not an atom, is the 'tot' value (sum"
            " of all atoms), this field always is included in the outfile and "
            "it always is the 'tot' value from infile, regardless the selection"
            " of atoms."
        )
        OptFilter.add_argument(
            "-a", "--atoms", type=int, nargs="+", action="append", help=phelp
        )

        phelp = (
            "List of orbitals to group as a single entry. Each group of "
            "orbitals needs a different `--orbitals` list. By instance, to "
            "group orbitals in 's','p', 'd' it is needed `-o 0 -o 1 2 3 -o 4 5 "
            "6 7 8`. Where 0=s, 1,2,3=px,py,pz, 4...9=dxx...dyz. Mind the last "
            "value (aka) 'tot' always is written."
        )
        OptFilter.add_argument(
            "-o", "--orbitals", help=phelp, type=int, nargs="+", action="append"
        )

        phelp = (
            "Keeps only the bands between `min` and `max` indexes. To keep the "
            "bands from 120 to 150 you should give `-b 120 150 `. It is not "
            "obvious which indexes are in the interest region, therefore I "
            "recommend you trial and error "
        )
        OptFilter.add_argument("-b", "--bands", help=phelp, type=int, nargs=2)

        phelp = (
            "Which spin components should be written: 0=density, 1,2,3=Sx,Sy,Sz."
            " They are not averaged."
        )
        OptFilter.add_argument("-s", "--spin", help=phelp, type=int, nargs="+")

        phelp = (
            "enable to give atoms list in a more human, 1-based order (say the"
            " 1st is 1, 2nd is 2 and so on ). Mind: this only holds for atoms."
        )
        parserFilter.add_argument("--human", help=phelp, action="store_true")

        phelp = (
            "List of names of new 'orbitals' to appear in the new file, eg. "
            "(`--orbital_names s p d` for a 's', 'p', 'd'). Only meaningful "
            "when manipulating the orbitals, ie: using `-o` "
        )
        parserFilter.add_argument("--orbital_names", help=phelp, nargs="+")

        parserFilter.set_defaults(func=call_filter)

        ################ fermi2D ##########################################
        parserFermi2D = subparsers.add_parser(
            "fermi2D",
            help="Plot the Fermi surface for a 2D Brillouin zone (layer-wise)"
            "along z and just perpendicular to z! (actually k_z)",
        )

        phelp = "Input file. It can be compressed"
        parserFermi2D.add_argument("file", help=phelp)

        phelp = (
            "Spin component to be used: for non-polarized calculations density "
            "is '-s 0'. For spin polarized case test '-s 0' (ignore the spin) or"
            " '-s 1' (assing a sign to the spin channel). For "
            "non-collinear stuff you can use '-s 0', '-s 1', '-s 2', -s "
            "3 for the magnitude, x, y, z components of your spin "
            "vector, in this case you really want to see spin textures "
            "'--st'. Default: s=0"
        )
        parserFermi2D.add_argument(
            "-s", "--spin", type=int, choices=[0, 1, 2, 3], default=0, help=phelp
        )

        phelp = (
            "List of atoms to be used (0-based): ie. '-a 0 2' to select the 1st"
            " and 3rd. It defaults to the last one (-a -1 the 'tot' entry)"
        )
        parserFermi2D.add_argument("-a", "--atoms", type=int, nargs="+", help=phelp)

        phelp = (
            "Orbital index(es) to be used 0-based. Take a look to the PROCAR "
            "file. Its default is the last field (ie: 'tot'). From a standard "
            "PROCAR: `-o 0`='s', `-o 1 2 3`='p', `-o 4 5 6 7 8`='d'."
        )
        parserFermi2D.add_argument("-o", "--orbitals", type=int, nargs="+", help=phelp)

        phelp = (
            "Energy for the surface. To plot the Fermi surface at Fermi Energy `-e 0`"
        )
        parserFermi2D.add_argument(
            "-e", "--energy", help=phelp, type=float, required=True
        )

        phelp = (
            "Set the Fermi energy (or any reference energy) as zero. To get it "
            "you should `grep E-fermi` the self-consistent OUTCAR. See `--outcar`"
        )
        parserFermi2D.add_argument("-f", "--fermi", help=phelp, type=float)

        phelp = (
            "reciprocal space basis vectors. 9 number are required b1x b1y ... "
            " b3z. This option is quite involved, so I recommend you to use `--outcar`"
        )
        parserFermi2D.add_argument("--rec_basis", help=phelp, type=float, nargs=9)

        phelp = (
            "Apply a rotational symmetry to unfold the Kpoints found. If your"
            "PROCAR only has a portion of the Brillouin Zone, you may want to "
            "plot the FULL BZ (ie: a Dirac cone at Gamma will look like a cone "
            "and not like a segment of circle). Supported rotations are "
            "1,2,3,4,6. All of them along Z and centered at Gamma. Consider to "
            "'--translate' your cell to rotate with other origin. This is the "
            "last symmetry operation to be performed."
        )
        parserFermi2D.add_argument("--rot_symm", help=phelp, type=int, default=1)

        phelp = (
            "Translate your mesh to the specified point. The point can be 3 "
            "coordinates (numbers) or the index of one K-point (zero-based, as "
            "usual). This is the first symmetry operation to be performed "
            "(i.e. rotations will take this point as the origin)."
        )
        parserFermi2D.add_argument(
            "--translate", help=phelp, nargs="+", default=[0, 0, 0]
        )

        phelp = (
            "A general rotation is applied to the data in the PROCAR. While this "
            " script has a large bias to work on the 'xy' plane, with this option"
            " you can rotate your whole PROCAR to fit the 'xy' plane. A rotation "
            "is composed by one angle plus one fixed axis, eg: '--rotation 90 1 0"
            " 0' is 90 degrees along the x axis, this changes the 'xy'->'xz'. The"
            " rotation is performed after the translation and before applying "
            "rot_symm. "
        )
        parserFermi2D.add_argument(
            "--rotation", help=phelp, type=float, nargs=4, default=[0, 0, 0, 1]
        )

        phelp = "enable to give atoms list in a more human, 1-based way (say the 1st is 1,2nd is 2 and so on )"
        parserFermi2D.add_argument("-u", "--human", help=phelp, action="store_true")

        phelp = "If set, masks(hides) values lowers than it. Useful to remove unwanted bands."
        parserFermi2D.add_argument("--mask", type=float, help=phelp)

        phelp = (
            "Saves the figure, instead of display it on screen. Anyway, you can save from"
            " the screen too. Any file extension supported by"
            "matplotlib.savefig is valid (if you are too lazy"
            " to google it, trial and error also works)"
        )
        parserFermi2D.add_argument("--savefig", help=phelp)

        phelp = (
            "OUTCAR file where to find the reciprocal lattice vectors and "
            "perhaps E_fermi. Mind: '--fermi' has precedence, remember that the"
            " E-fermi should correspond to a self-consistent run, not a "
            "bandstructure! (however, this is irrelevant for basis vectors)"
        )
        parserFermi2D.add_argument("--outcar", help=phelp)

        phelp = (
            "Plot of the spin texture (ie: spin arrows) on the Fermi's surface."
            " This option works quite indepentent of another options."
        )
        parserFermi2D.add_argument("--st", help=phelp, action="store_true")

        phelp = (
            "Plot of the spin texture without arrows (just intensity) for a "
            "given spin direction on the Fermi's surface.  This option works"
            "quite indepentent of another options but needs to set '--st' and '--spin'."
        )
        parserFermi2D.add_argument("--noarrow", help=phelp, action="store_true")

        parserFermi2D.set_defaults(func=call_fermi2D)

        ################ Fermi3D ##########################################

        parserfermi3D = subparsers.add_parser("fermi3D", help="Plot 3D Fermi surface")
        parserfermi3D.add_argument("procar", help="PROCAR file.")
        parserfermi3D.add_argument("outcar", help="OUTCAR file.")
        parserfermi3D.add_argument(
            "bands", help="Array of bands to be included. -1 considers all.", default=-1
        )
        parserfermi3D.add_argument(
            "scale", help="Interpolation factor", type=float, default=1
        )
        parserfermi3D.add_argument(
            "mode", help="Plot mode.", choices=["plain", "parametric", "external"]
        )
        parserfermi3D.add_argument(
            "-st", help="Flag to set spin texture on.", action="store_true"
        )
        parserfermi3D.add_argument("kwargs", help="Additional arguments.", nargs="*")
        parserfermi3D.set_defaults(func=call_fermi3D)

        ################# repair ##########################################
        parserrepair = subparsers.add_parser(
            "repair", help="Repairs formatting issues in PROCAR file."
        )
        parserrepair.add_argument("infile", help="Input file. Can be compressed.")
        parserrepair.add_argument("outfile", help="Output file.")
        parserrepair.set_defaults(func=call_repair)

        ################# bandgap ##########################################
        parserbandgap = subparsers.add_parser(
            "bandgap",
            help="Calculate bandgap. procar and outcar needed only for Abinit and VASP.",
        )
        parserbandgap.add_argument("procar", help="PROCAR file.")
        parserbandgap.add_argument("outcar", help="OUTCAR file.")
        parserbandgap.add_argument(
            "code",
            help="code",
            choices=["vasp", "qe", "lobster", "abinit", "elk"],
            default="vasp",
        )
        parserbandgap.add_argument(
            "fermi", help="Fermi energy. Retrived from output if not provided."
        )
        parserbandgap.set_defaults(func=call_bandgap)

        ################## k-mesh ########################################
        parsergenerate2dkmesh = subparsers.add_parser(
            "generate2dkmesh",
            help="Generate a 2D k-mesh"
            "centered at a given k-point in a given k-plane.",
        )
        parsergenerate2dkmesh.add_argument("x1", help="x1 coordinate")
        parsergenerate2dkmesh.add_argument("y1", help="y1 coordinate")
        parsergenerate2dkmesh.add_argument("x2", help="x2 coordinate")
        parsergenerate2dkmesh.add_argument("y2", help="y2 coordinate")
        parsergenerate2dkmesh.add_argument("z", help="z plane")
        parsergenerate2dkmesh.add_argument(
            "nkx", help="number of grids in the x direction"
        )
        parsergenerate2dkmesh.add_argument(
            "nky", help="number of grids in the y direction"
        )
        parsergenerate2dkmesh.set_defaults(func=call_generate2dkmesh)

        ################## k-path ####################################################
        parserkpath = subparsers.add_parser(
            "kpath", help="k-path generator.", formatter_class=RawTextHelpFormatter
        )
        parserkpath.add_argument("infile", help="POSCAR file", default="POSCAR")
        parserkpath.add_argument("-grid_size", help="Grid size", default=40, type=int)
        parserkpath.add_argument(
            "-with_time_reversal",
            help="Flag to turn on time reversal symmetry",
            action="store_true",
        )
        parserkpath.add_argument(
            "-recipe",
            help="The algorithm that defines the special points and paths",
            type=str,
            default="hpkot",
        )
        parserkpath.add_argument(
            "-threshold",
            help="The threshold to use to verify if we are in an edge case",
            type=float,
            default=1e-07,
        )
        parserkpath.add_argument(
            "-symprec",
            help="The symmetry precision used internally by SPGLIB",
            type=float,
            default=1e-05,
        )
        parserkpath.add_argument(
            "-angle_tolerence",
            help="Angle_tolerance used internally by SPGLIB",
            type=float,
            default=-1.0,
        )
        parserkpath.add_argument(
            "-supercell_matrix",
            help="The super cell for band unfolding. Default 3x3 identity matrix.",
            type=int,
            default=np.eye(3),
        )
        parserkpath.set_defaults(func=call_kpath)

        ################### bandstructure ######################################################
        parserBandsplot = subparsers.add_parser(
            "bandsplot",
            help="Bandstructure plot.",
            formatter_class=RawTextHelpFormatter,
        )

        phelp = "Input file. It can be compressed"
        parserBandsplot.add_argument("file", help=phelp)

        phelp = (
            "Plot mode:\n"
            "-m  scatter : is a points plot with the color given by the chosen\n"
            "  projection. It produces a rather heavy pdf file.\n\n"
            "-m  parametric : like scatter, but with lines instead of points \n"
            "  (bands crossings are not handled, and some  unphysical 'jumps' \n"
            " can be present). Sligthy smaller PDF size.\n\n"
            "-m plain : is a featureless bandstructure ignoring all data about\n"
            "  projections. Rather light-weight\n\n"
            "-m atomic : For non-periodic system, like molecules, rather ugly \n"
            "  but useful to visualize energy level. Only 1 K-point!\n\n"
        )
        choices = ["scatter", "plain", "parametric", "atomic"]
        parserBandsplot.add_argument(
            "-m", "--mode", help=phelp, default="plain", choices=choices
        )

        phelp = "Color of the bands for plain mode."
        parserBandsplot.add_argument("-color", help=phelp, default="blue")

        phelp = "Name of Abinit output file if used."
        parserBandsplot.add_argument("-abinit", help=phelp, default=None)

        phelp = (
            "Spin component to be used (default -s 0): \n\n"
            "Non-polarized calculations density is '-s 0', just ignore it.\n\n"
            "Spin-Polarized (collinear) calculation: \n"
            "-s 0 are the unpolarized bands, the spin-polarization is ignored.\n"
            "-s 1 'spin-polarized' bands, the character of 'up' bands positive,\n"
            "  but negative for 'down' bands, this means that the color of \n"
            "  'down' is negative. Useful together with '--cmap seismic'.\n\n"
            "Non-collinear calculation: \n"
            "-s 0 : density, ie: Spin-orbit-coupling but don't care of spin.\n"
            "-s 1 : Sx, projection along 'x' quantization axis, see SAXIS flag\n"
            "  in the VASP manual\n"
            "-s 2 : Sy, projection along 'y' quantization axis.\n"
            "-s 3 : Sy, projection along 'z' quantization axis.\n"
            "-s st : Spin-texture perpendicular in the plane (kx,ky) to each\n "
            "(kx,ky) vector. Useful for Rashba-like states in surfaces. Use\n "
            "'--cmap seismic'\n\n "
        )
        parserBandsplot.add_argument(
            "-s", "--spin", choices=["0", "1", "2", "3", "st"], default="0", help=phelp
        )

        phelp = (
            "List of rows (atoms) to be used. This list refers to the rows of\n"
            "(each block of) your PROCAR file. If you haven't manipulated your\n"
            "PROCAR (eg: with the '-a' option of 'filter' mode) each row\n"
            "correspond to the respective atom in the POSCAR.\n\n"
            "Mind: This list is 0-based, ie: the 1st atom is 0, the 2nd is 1,\n"
            "  and so on. If you need to be treated like a human, specify '-u'\n"
            "or '--human' and 1st->1, 2nd->2, etc.\n\n"
            "Example:\n"
            "-a 0 2 :  select the 1st  and 3rd. rows (likely 1st and 3rd atoms)"
            "\n\n"
        )
        parserBandsplot.add_argument(
            "-a", "--atoms", type=int, nargs="+", help=phelp, default=None
        )

        phelp = (
            "Orbitals index(es) to be used, take a look to the PROCAR file, \n"
            "they are 's py pz px ...', then s->0, py->1, pz->2 and so on. \n"
            "Note that indexes begin at 0!. Its default is the last field (ie:\n"
            "'tot', did you saw the PROCAR?). Some examples:\n\n"
            "-o 0 : s-orbital (unless you modified the orbitals, eg. 'filter')\n"
            "-o 1 2 3 : py+pz+px (unless you modified the orbitals)\n"
            "-o 4 5 6 7 8 : all the d-orbitasl (unless...)\n"
            "-o 2 6 : pz+dzz (did you look at the PROCAR?)\n\n "
        )
        parserBandsplot.add_argument(
            "-o", "--orbitals", type=int, nargs="+", help=phelp, default=None
        )

        phelp = (
            "Set the Fermi energy (or any reference energy) as the zero energy.\n"
            "See '--outcar', avoids to give it explicitly. A list of \n"
            "k-dependant 'fermi-like energies' are also accepted (useful to\n"
            "compare different systems in one PROCAR made by hand). \n\n"
            "Mind: The Fermi energy MUST be the one from the self-consistent\n"
            "calculation, not from a Bandstructure calculation!\n\n"
        )
        parserBandsplot.add_argument(
            "-f", "--fermi", type=float, help=phelp, nargs="+", default=None
        )

        phelp = (
            "Min/Max energy to be ploted. Example:\n "
            "--elimit -1 1 : From -1 to 1 around Fermi energy (if given)\n\n"
        )
        parserBandsplot.add_argument(
            "--elimit", type=float, nargs=2, help=phelp, default=None
        )

        phelp = (
            "If given, it masks(hides) bands with values lowers than 'mask'.\n"
            "It is useful to remove 'unwanted' bands. For instance, if you\n"
            "project the bandstructure on a 'surface' atom -with the default\n"
            "colormap- some white points can appear, they are bands with \n"
            "almost no contribution to the 'surface': no physics but they \n"
            "still look ugly, to hide those bands use '--mask 0.08' (or some \n"
            "other small value). Mind: it works with the absolute value of\n"
            "projection (no problem with spin polarization)\n\n"
        )
        parserBandsplot.add_argument("--mask", type=float, help=phelp, default=None)

        phelp = (
            "Size of markers, if used. Each mode has it own scale,\n"
            "just test them\n\n"
        )
        parserBandsplot.add_argument(
            "--markersize", type=float, help=phelp, default=0.02
        )

        phelp = (
            "Change the color scheme. Example:\n\n"
            "--cmap  seismic : blue->white->red, useful to see the \n"
            "  spin-polarization of a band (it will blueish or reddish)\n"
            "  depending of spin channel\n"
            "--cmap  seismic_r : the 'seismic' colormap, but reversed.\n\n"
        )
        parserBandsplot.add_argument("--cmap", help=phelp, default="jet")

        phelp = (
            "Do you want to Normalize the plots to the same scale of colors\n"
            "(ie: the numbers on the bar at the right), just try '--vmax'\n\n"
            "--vmax 1 : If you are looking for the s, p or d character.\n"
            "--vmax 0.2: If you want to capture some tiny effect, eg: s-band of\n"
            "  a impurity on a metal\n\n"
        )
        parserBandsplot.add_argument("--vmax", type=float, help=phelp, default=None)

        phelp = (
            "Like '--vmax' (see '--vmax'), However, for spin-polarized \n"
            "(collinear or not) you can set it to a negative value. Actually\n"
            "you can do it for a non-spin-polarized calculation and the \n"
            "effect will be a 'stretching' of the color scheme, try it.\n\n"
        )
        parserBandsplot.add_argument("--vmin", type=float, help=phelp, default=None)

        phelp = "switch on/off the grid. Default is 'on'\n\n"
        parserBandsplot.add_argument("--grid", type=bool, help=phelp, default=True)

        phelp = (
            "set the marker shape, ie: 'o'=circle, 's'=square,\ '-'=line\n"
            "(only mode `plain`, other symbols: google pyplot markers)\n\n"
        )
        parserBandsplot.add_argument("--marker", type=str, help=phelp, default="o")

        phelp = (
            "Some fault tolerance for ill-formatted files (stupid fortran)\n"
            "But be careful, something could be messed up and don't work (at\n"
            "least as expected). Length of K-points paths will be ignored\n\n"
        )
        parserBandsplot.add_argument("--permissive", help=phelp, action="store_true")

        phelp = (
            "Enable human-like 1-based order (ie 1st is 1, 2nd is 2, and so\n"
            "on). Mind: this only works for atoms, not for orbitals or spin\n\n"
        )
        parserBandsplot.add_argument("-u", "--human", help=phelp, action="store_true")

        phelp = (
            "Saves the figure, instead of display it on screen. Anyway, you can\n"
            "save from the screen too. Any file extension supported by\n "
            "`matplotlib.savefig` is valid (if you are too lazy to google it,\n"
            "trial and error also works fine)\n\n"
        )
        parserBandsplot.add_argument("--savefig", help=phelp, default=None)

        phelp = (
            "list of ticks along the kpoints axis (x axis). For instance a\n"
            "bandstructure G-X-M with 10 point by segment should be:\n "
            "--kticks 0 9 19\n\n"
        )
        parserBandsplot.add_argument("--kticks", help=phelp, nargs="+", type=int)

        phelp = (
            "Names of the points given in `--kticks`. In the `kticks` example\n"
            'they should be `--knames "\$Gamma\$" X M`. As you can see \n'
            "LaTeX stuff works with a minimal mess (extra \\s)\n\n"
        )
        parserBandsplot.add_argument(
            "--knames", help=phelp, nargs="+", type=str, default=None
        )

        phelp = (
            "Title, to use several words, use quotation marks\"\" or ''. Latex\n"
            " works if you scape the special characteres, ie: $\\alpha$ -> \n"
            "\$\\\\alpha\$"
        )
        parserBandsplot.add_argument(
            "-t", "--title", help=phelp, type=str, default=None
        )

        phelp = (
            "OUTCAR file where to find the reciprocal lattice vectors and\n "
            "perhaps E_fermi.\n"
            "Mind: '--fermi' has precedence, remember that the E-fermi should\n"
            "correspond to a self-consistent run, not to a bandstructure!\n"
            "(however, the basis and reciprocal vectors will be safe from the\n"
            "non-self-consistentness)\n\n"
        )
        parserBandsplot.add_argument("--outcar", help=phelp, default=None)

        phelp = "KPOINTS file for bandstructure plotting.\n"
        parserBandsplot.add_argument("--kpointsfile", help=phelp, default=None)

        phelp = "Convert k-points from reduced to cartesian for plot #1?"
        parserBandsplot.add_argument("-kdirect", help=phelp, action="store_false")

        parserBandsplot.set_defaults(func=call_bandsplot)

        args = parser.parse_args()
        args.func(args)

    else:
        print("PyProcar: A Python library for analyzing PROCAR files.\n")
        print("Usage: procar [-h]")
        print(
            "{cat,mergeabinit,unfold,filter,fermi2D,fermi3D,repair,generate2dkmesh,kpath,bandsplot,bandscompare}"
        )



import numpy as np
from .abinitparser import AbinitParser
from .elkparser import ElkParser
from .procarparser import ProcarParser
from .qeparser import QEParser
from .lobsterparser import LobsterParser
from .utilsprocar import UtilsProcar


def getFermi(procar, code, outcar):  # from ScriptsBandPlot made into method
    fermi = None

    if code == "vasp":
        # Parses through Bands in PROCAR
        procarFile = ProcarParser()
        procarFile.readFile(procar=procar)
        if outcar:
            outcarparser = UtilsProcar()
            if fermi is None:
                fermi = outcarparser.FermiOutcar(outcar)
                print("Fermi energy found in OUTCAR file = %s eV" % str(fermi))
        else:
            print("ERROR: OUTCAR Not Found")

    elif code == "elk":
        if fermi is None:
            procarFile = ElkParser()
            procarFile.readFile()
            fermi = procarFile.fermi
            print("Fermi energy found in Elk output file = %s eV " % str(fermi))

    elif code == "abinit":
        procarFile = ProcarParser()
        procarFile.readFile(procar=procar)
        if fermi is None:
            abinitFile = AbinitParser(abinit_output=outcar)
            fermi = abinitFile.fermi
            print("Fermi energy found in Abinit output file = %s eV" % str(fermi))

    elif code == "qe":
        if fermi is None:
            procarFile = QEParser()
            fermi = procarFile.fermi
            print("Fermi energy   :  %s eV (from Quantum Espresso output)" % str(fermi))

    elif code == "lobster":
        if fermi is None:
            fermi = procarFile.fermi
            print("Fermi energy   :  %s eV (from Lobster output)" % str(fermi))
            # lobster already shifts fermi so we set it to zero here.
            fermi = 0.0

    return fermi


def bandgap(procar=None, outcar=None, code="vasp", fermi=None):

    bandGap = None

    if fermi is None:
        fermi = getFermi(procar, code, outcar)

    if code == "vasp":
        procarFile = ProcarParser()
        procarFile.readFile(procar=procar)

    elif code == "abinit":
        procarFile = ProcarParser()
        procarFile.readFile(procar=procar)

    elif code == "elk":
        procarFile = ElkParser()

    elif code == "qe":
        procarFile = QEParser()

    elif code == "lobsters":
        procarFile = LobsterParser()

    bands = np.array(procarFile.bands)
    subBands = np.subtract(bands, fermi)

    negArr = subBands[subBands < 0]
    posArr = subBands[subBands > 0]

    negVal = np.amax(negArr)
    posVal = np.amin(posArr)

    idx = np.where(subBands == negVal)[1][0]

    if all(i >= 0 for i in subBands[:, idx]) or all(i <= 0 for i in subBands[:, idx]):
        possibleGap = posVal - negVal
        if bandGap is None:
            bandGap = possibleGap
        elif possibleGap < bandGap:
            bandGap = possibleGap
    else:
        bandGap = 0

    print("Band Gap = %s eV " % str(bandGap))

    return bandGap



import re

import matplotlib.pyplot as plt
import numpy as np

from .abinitparser import AbinitParser
from .elkparser import ElkParser
from .qeparser import QEParser
from .lobsterparser import LobsterParser
from .procarparser import ProcarParser
from .procarplot import ProcarPlot
from .procarselect import ProcarSelect
from .splash import welcome
from .utilsprocar import UtilsProcar

# import matplotlib
plt.rcParams["mathtext.default"] = "regular"  # Roman ['rm', 'cal', 'it', 'tt', 'sf',
#                                                   'bf', 'default', 'bb', 'frak',
#                                                   'circled', 'scr', 'regular']
plt.rcParams["font.family"] = "Georgia"
plt.rc("font", size=22)  # controls default text sizes
plt.rc("axes", titlesize=22)  # fontsize of the axes title
plt.rc("axes", labelsize=22)  # fontsize of the x and y labels
plt.rc("xtick", labelsize=22)  # fontsize of the tick labels
plt.rc("ytick", labelsize=22)  # fontsize of the tick labels
# plt.rc('legend', fontsize=22)    # legend fontsize
# plt.rc('figure', titlesize=22)  # fontsize of the figure title


def bandsplot(
    procarfile=None,
    mode="plain",
    color="blue",
    abinit_output=None,
    spin=0,
    atoms=None,
    orbitals=None,
    fermi=None,
    elimit=[-2, 2],
    mask=None,
    markersize=0.02,
    cmap="jet",
    vmax=None,
    vmin=None,
    grid=True,
    marker="o",
    permissive=False,
    human=False,
    savefig=None,
    kticks=None,
    knames=None,
    title=None,
    outcar=None,
    kpointsfile=None,
    kdirect=True,
    code="vasp",
    separate=False,
    ax=None,
    discontinuities=None,
    show=True,
    lobstercode="qe",
):

    """This function plots band structures
  """
    welcome()

    # Turn interactive plotting off
    plt.ioff()

    # Verbose section

    # First handling the options, to get feedback to the user and check
    # that the input makes sense.
    # It is quite long
    if atoms is None:
        atoms = [-1]
        if human is True:
            print("WARNING: `--human` option given without atoms list!")
            print("--human will be set to False (ignored)\n ")
            human = False
    if orbitals is None:
        orbitals = [-1]

    print("Script initiated")
    print("code           : ", code)
    print("input file     : ", procarfile)
    print("mode           : ", mode)
    print("spin comp.     : ", spin)
    print("atoms list     : ", atoms)
    print("orbs. list     : ", orbitals)

    if (
        fermi is None
        and outcar is None
        and abinit_output is None
        and (code != "elk" and code != "qe" and code != "lobster")
    ):
        print(
            "WARNING : Fermi Energy not set! Please set manually or provide output file and set code type."
        )
        fermi = 0

    elif fermi is None and code == "elk":
        fermi = None

    elif fermi is None and code == "qe":
        fermi = None

    elif fermi is None and code == "lobster":
        fermi = None

    print("fermi energy   : ", fermi)
    print("energy range   : ", elimit)

    if mask is not None:
        print("masking thres. : ", mask)

    print("colormap       : ", cmap)
    print("markersize     : ", markersize)
    print("permissive     : ", permissive)
    if permissive:
        print("INFO : Permissive flag is on! Be careful")
    print("vmax           : ", vmax)
    print("vmin           : ", vmin)
    print("grid enabled   : ", grid)
    if human is not None:
        print("human          : ", human)
    print("savefig        : ", savefig)
    print("title          : ", title)
    print("outcar         : ", outcar)

    if kdirect:
        print("k-grid         :  reduced")
    else:
        print(
            "k-grid         :  cartesian (Remember to provide an output file for this case to work.)"
        )

    if discontinuities is None:
        discontinuities = []

    #### READING KPOINTS FILE IF PRESENT ####

    # If KPOINTS file is given:
    if kpointsfile is not None:
        # Getting the high symmetry point names from KPOINTS file
        f = open(kpointsfile)
        KPread = f.read()
        f.close()

        KPmatrix = re.findall("reciprocal[\s\S]*", KPread)
        tick_labels = np.array(re.findall("!\s(.*)", KPmatrix[0]))
        knames = []
        knames = [tick_labels[0]]

        ################## Checking for discontinuities ########################
        discont_indx = []
        icounter = 1
        while icounter < len(tick_labels) - 1:
            if tick_labels[icounter] == tick_labels[icounter + 1]:
                knames.append(tick_labels[icounter])
                icounter = icounter + 2
            else:
                discont_indx.append(icounter)
                knames.append(tick_labels[icounter] + "|" + tick_labels[icounter + 1])
                icounter = icounter + 2
        knames.append(tick_labels[-1])
        discont_indx = list(dict.fromkeys(discont_indx))

        ################# End of discontinuity check ##########################

        # Added by Nicholas Pike to modify the output of seekpath to allow for
        # latex rendering.
        for i in range(len(knames)):
            if knames[i] == "GAMMA":
                knames[i] = "\Gamma"
            else:
                pass

        knames = [str("$" + latx + "$") for latx in knames]

        # getting the number of grid points from the KPOINTS file
        f2 = open(kpointsfile)
        KPreadlines = f2.readlines()
        f2.close()
        numgridpoints = int(KPreadlines[1].split()[0])

        kticks = [0]
        gridpoint = 0
        for kt in range(len(knames) - 1):
            gridpoint = gridpoint + numgridpoints
            kticks.append(gridpoint - 1)

        print("knames         : ", knames)
        print("kticks         : ", kticks)

        # creating an array for discontunuity k-points. These are the indexes
        # of the discontinuity k-points.
        for k in discont_indx:
            discontinuities.append(kticks[int(k / 2) + 1])
        if discontinuities:
            print("discont. list  : ", discontinuities)

    #### END OF KPOINTS FILE DEPENDENT SECTION ####

    # The spin argument should be a number (index of an array), or
    # 'st'. In the last case it will be handled separately (later)

    spin = {"0": 0, "1": 1, "2": 2, "3": 3, "st": "st"}[str(spin)]

    #### parsing the PROCAR file or equivalent to retrieve spd data ####
    code = code.lower()
    if code == "vasp":
        procarFile = ProcarParser()

    elif code == "abinit":
        procarFile = ProcarParser()
        abinitFile = AbinitParser(abinit_output=abinit_output)

    elif code == "elk":
        # reciprocal lattice already taken care of
        procarFile = ElkParser(kdirect=kdirect)

        # Retrieving knames and kticks from Elk
        if kticks is None and knames is None:
            if procarFile.kticks and procarFile.knames:
                kticks = procarFile.kticks
                knames = procarFile.knames

    elif code == "qe":
        # reciprocal lattice already taken care of
        procarFile = QEParser(kdirect=kdirect)

        # Retrieving knames and kticks from QE
        if kticks is None and knames is None:
            if procarFile.kticks and procarFile.knames:
                kticks = procarFile.kticks
                knames = procarFile.knames

            # Retrieving discontinuities if present
            if procarFile.discontinuities:
                discontinuities = procarFile.discontinuities

    elif code == "lobster":
        # reciprocal lattice already taken care of
        procarFile = LobsterParser(kdirect=kdirect, lobstercode=lobstercode)

        # Retrieving knames and kticks from Lobster
        if kticks is None and knames is None:
            if procarFile.kticks and procarFile.knames:
                kticks = procarFile.kticks
                knames = procarFile.knames

            # Retrieving discontinuities if present
            if procarFile.discontinuities:
                discontinuities = procarFile.discontinuities

    # If ticks and names are given by the user manually:
    if kticks is not None and knames is not None:
        ticks = list(zip(kticks, knames))
    elif kticks is not None:
        ticks = list(zip(kticks, kticks))
    else:
        ticks = None

    if kpointsfile is None and kticks and knames:
        print("knames         : ", knames)
        print("kticks         : ", kticks)
        if discontinuities:
            print("discont. list  : ", discontinuities)

    # The second part of this function is parse/select/use the data in
    # OUTCAR (if given) and PROCAR

    # first parse the outcar if given, to get Efermi and Reciprocal lattice
    recLat = None
    if code == "vasp":
        if outcar:
            outcarparser = UtilsProcar()
            if fermi is None:
                fermi = outcarparser.FermiOutcar(outcar)
                print("Fermi energy   :  %s eV (from OUTCAR)" % str(fermi))
            recLat = outcarparser.RecLatOutcar(outcar)

    elif code == "elk":
        if fermi is None:
            fermi = procarFile.fermi
            print("Fermi energy    :  %s eV (from Elk output)" % str(fermi))

    elif code == "qe":
        if fermi is None:
            fermi = procarFile.fermi
            print("Fermi energy   :  %s eV (from Quantum Espresso output)" % str(fermi))

    elif code == "lobster":
        if fermi is None:
            fermi = procarFile.fermi
            print("Fermi energy   :  %s eV (from Lobster output)" % str(fermi))
            # lobster already shifts fermi so we set it to zero here.
            fermi = 0.0

    elif code == "abinit":
        if fermi is None:
            fermi = abinitFile.fermi
            print("Fermi energy   :  %s eV (from Abinit output)" % str(fermi))
        recLat = abinitFile.reclat

    # if kdirect = False, then the k-points will be in cartesian coordinates.
    # The output should be read to find the reciprocal lattice vectors to transform
    # from direct to cartesian

    if code == "vasp":
        if kdirect:
            procarFile.readFile(procarfile, permissive)
        else:
            procarFile.readFile(procarfile, permissive, recLattice=recLat)

    elif code == "abinit":
        if kdirect:
            procarFile.readFile(procarfile, permissive)
        else:
            procarFile.readFile(procarfile, permissive, recLattice=recLat)

    # processing the data, getting an instance of the class that reduces the data
    data = ProcarSelect(procarFile, deepCopy=True, mode=mode)
    numofbands = int(data.spd.shape[1] / 2)

    # handling the spin, `spin='st'` is not straightforward, needs
    # to calculate the k vector and its normal. Other `spin` values
    # are trivial.
    if spin == "st":
        # two `ProcarSelect` instances, to store temporal values: spin_x, spin_y
        dataX = ProcarSelect(procarFile, deepCopy=True)
        dataX.selectIspin([1])
        dataX.selectAtoms(atoms, fortran=human)
        dataX.selectOrbital(orbitals)
        dataY = ProcarSelect(procarFile, deepCopy=True)
        dataY.selectIspin([2])
        dataY.selectAtoms(atoms, fortran=human)
        dataY.selectOrbital(orbitals)
        # getting the signed angle of each K-vector
        angle = np.arctan2(dataX.kpoints[:, 1], (dataX.kpoints[:, 0] + 0.000000001))
        sin = np.sin(angle)
        cos = np.cos(angle)
        sin.shape = (sin.shape[0], 1)
        cos.shape = (cos.shape[0], 1)
        # print sin, cos
        # storing the spin projection into the original array
        data.spd = -sin * dataX.spd + cos * dataY.spd
    else:
        data.selectIspin([spin], separate=separate)
        data.selectAtoms(atoms, fortran=human)
        data.selectOrbital(orbitals)

    # Plotting the data
    if separate:
        if spin == 0:
            # plotting spin up bands separately
            data.bands = (
                data.bands[:, :numofbands].transpose() - np.array(fermi)
            ).transpose()
            print("Plotting spin up bands...")

        elif spin == 1:
            # plotting spin down bands separately
            data.bands = (
                data.bands[:, numofbands:].transpose() - np.array(fermi)
            ).transpose()
            print("Plotting spin down bands...")

        plot = ProcarPlot(data.bands, data.spd, data.kpoints)

    else:
        # Regular plotting method. For spin it plots density or magnetization.
        data.bands = (data.bands.transpose() - np.array(fermi)).transpose()
        plot = ProcarPlot(data.bands, data.spd, data.kpoints)

    ###### start of mode dependent options #########

    if mode == "scatter":
        fig, ax1 = plot.scatterPlot(
            mask=mask,
            size=markersize,
            cmap=cmap,
            vmin=vmin,
            vmax=vmax,
            marker=marker,
            ticks=ticks,
            discontinuities=discontinuities,
            ax=ax,
        )
        if fermi is not None:
            ax1.set_ylabel(r"$E-E_f$ [eV]")
        else:
            ax1.set_ylabel(r"Energy [eV]")
        if elimit is not None:
            ax1.set_ylim(elimit)

    elif mode == "plain":
        fig, ax1 = plot.plotBands(
            markersize,
            marker=marker,
            ticks=ticks,
            color=color,
            discontinuities=discontinuities,
            ax=ax,
        )
        if fermi is not None:
            ax1.set_ylabel(r"$E-E_f$ [eV]")
        else:
            ax1.set_ylabel(r"Energy [eV]")
        if elimit:
            ax1.set_ylim(elimit)

    elif mode == "parametric":
        fig, ax1 = plot.parametricPlot(
            cmap=cmap,
            vmin=vmin,
            vmax=vmax,
            mask=mask,
            ticks=ticks,
            discontinuities=discontinuities,
            ax=ax,
        )
        if fermi is not None:
            ax1.set_ylabel(r"$E-E_f$ [eV]")
        else:
            ax1.set_ylabel(r"Energy [eV]")
        if elimit is not None:
            ax1.set_ylim(elimit)

    elif mode == "atomic":
        fig, ax1 = plot.atomicPlot(cmap=cmap, vmin=vmin, vmax=vmax, ax=ax)
        if fermi is not None:
            ax1.set_ylabel(r"$E-E_f$ [eV]")
        else:
            ax1.set_ylabel(r"Energy [eV]")
        if elimit is not None:
            ax1.set_ylim(elimit)
    ###### end of mode dependent options ###########

    if grid:
        ax1.grid()

    if title:
        ax1.set_title(title)

    else:
        if savefig:
            plt.savefig(savefig, bbox_inches="tight")
            plt.close()  # Added by Nicholas Pike to close memory issue of looping and creating many figures
            return None, None
        else:
            if show:
                plt.show()
            else:
                pass
        return fig, ax1


# if __name__ == "__main__":
# bandsplot(mode='parametric',elimit=[-6,6],orbitals=[4,5,6,7,8],vmin=0,vmax=1, code='elk')
# knames=['$\Gamma$', '$X$', '$M$', '$\Gamma$', '$R$','$X$'],
# kticks=[0, 8, 16, 24, 38, 49])



from re import findall
from numpy import array


class AbinitParser:
    """
    This class contains methods to parse the fermi energy and reciprocal
    lattice vectors from the Abinit output file.
    """

    def __init__(self, abinit_output=None):

        self.abinit_output = abinit_output
        self.fermi = None
        self.reclat = None  # reciprocal lattice vectors

        self._readFermi()
        self._readRecLattice()

        return

    def _readFermi(self):
        rf = open(self.abinit_output, "r")
        data = rf.read()
        rf.close()
        self.fermi = float(
            findall("Fermi\w*.\(\w*.HOMO\)\s*\w*\s*\(\w*\)\s*\=\s*([0-9.+-]*)", data)[0]
        )

    def _readRecLattice(self):
        rf = open(self.abinit_output, "r")
        data = rf.read()
        rf.close()
        lattice_block = findall(r"G\([1,2,3]\)=\s*([0-9.\s-]*)", data)
        lattice_block = lattice_block[3:]
        self.reclat = array(
            [lattice_block[0:3][i].split() for i in range(len(lattice_block))],
            dtype=float,
        )



#!/usr/bin/env python
import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.collections import LineCollection
from matplotlib.colors import colorConverter
import numpy as np
import re
import sys
import argparse
import os.path


def plot_band_weight(
    kslist,
    ekslist,
    wkslist=None,
    efermi=None,
    shift_efermi=False,
    yrange=None,
    output=None,
    style="alpha",
    color="blue",
    axis=None,
    width=10,
    fatness=4,
    xticks=None,
    cmap=mpl.cm.bwr,
    weight_min=-0.1,
    weight_max=0.6,
):
    if axis is None:
        fig, a = plt.subplots()
    else:
        a = axis
    if efermi is not None and shift_efermi:
        ekslist = np.array(ekslist) - efermi
    else:
        ekslist = np.array(ekslist)

    xmax = max(kslist[0])
    if yrange is None:
        yrange = (
            np.array(ekslist).flatten().min() - 0.66,
            np.array(ekslist).flatten().max() + 0.66,
        )
    if wkslist is not None:
        for i in range(len(kslist)):
            x = kslist[i]
            y = ekslist[i]
            # lwidths=np.ones(len(x))
            points = np.array([x, y]).T.reshape(-1, 1, 2)
            segments = np.concatenate([points[:-1], points[1:]], axis=1)
            if style == "width":
                lwidths = np.array(wkslist[i]) * width
                lc = LineCollection(segments, linewidths=lwidths, colors=color)
            elif style == "alpha":
                lwidths = np.array(wkslist[i]) * width

                # The alpha values sometimes goes above 1 so in those cases we will normalize
                # the alpha values. -Uthpala
                alpha_values = [lwidth / (width + 0.05) for lwidth in lwidths]
                if max(alpha_values) > 1:
                    print("alpha is larger than 1. Renormalizing values.")
                    alpha_values = [
                        alpha_i / max(alpha_values) for alpha_i in alpha_values
                    ]

                lc = LineCollection(
                    segments,
                    linewidths=[fatness] * len(x),
                    colors=[
                        colorConverter.to_rgba(color, alpha=alpha_i)
                        for alpha_i in alpha_values
                    ],
                )

            elif style == "color" or style == "colormap":
                lwidths = np.array(wkslist[i]) * width
                norm = mpl.colors.Normalize(vmin=weight_min, vmax=weight_max)
                # norm = mpl.colors.SymLogNorm(linthresh=0.03,vmin=weight_min, vmax=weight_max)
                m = mpl.cm.ScalarMappable(norm=norm, cmap=cmap)
                # lc = LineCollection(segments,linewidths=np.abs(norm(lwidths)-0.5)*1, colors=[m.to_rgba(lwidth) for lwidth in lwidths])
                lc = LineCollection(
                    segments,
                    linewidths=lwidths,
                    colors=[m.to_rgba(lwidth) for lwidth in lwidths],
                )
            a.add_collection(lc)
    if axis is None:
        for ks, eks in zip(kslist, ekslist):
            a.plot(ks, eks, color="gray", linewidth=0.01)
        # a.set_xlim(0, xmax)
        # a.set_ylim(yrange)
        if xticks is not None:
            a.set_xticks(xticks[1])
            a.set_xticklabels(xticks[0])
            for x in xticks[1]:
                a.axvline(x, alpha=0.6, color="black", linewidth=0.7)
        if efermi is not None:
            if shift_efermi:
                a.axhline(linestyle="--", color="black")
            else:
                a.axhline(efermi, linestyle="--", color="black")

    return a


def main():
    parser = argparse.ArgumentParser(description="plot wannier bands.")
    parser.add_argument("fname", type=str, help="dat filename")
    parser.add_argument("-e", "--efermi", type=float, help="Fermi energy", default=None)
    parser.add_argument(
        "-o", "--output", type=str, help="output filename", default=None
    )
    parser.add_argument(
        "-w", "--weight", action="store_true", help="use -w to plot weighted band."
    )
    parser.add_argument(
        "-y", "--yrange", type=float, nargs="+", help="range of yticks", default=None
    )
    parser.add_argument(
        "-s", "--style", type=str, help="style of line, width | alpha", default="width"
    )
    args = parser.parse_args()
    if args.output is None:
        output = os.path.splitext(args.fname)[0] + ".png"
    if args.efermi is None:
        efermi = get_fermi("SCF/OUTCAR")
    plot_band_weight_file(
        fname=args.fname,
        efermi=efermi,
        weight=args.weight,
        yrange=args.yrange,
        style=args.style,
    )
    if output is not None:
        plt.savefig(output)

    plt.show()


if __name__ == "__main__":
    main()



from distutils.core import Extension, setup

setup(
    name="pyprocar",
    version="5.5.1",
    author="Francisco Munoz,Aldo Romero,Sobhit Singh,Uthpala Herath,Pedram Tavadze,Eric Bousquet, Xu He",
    author_email="fvmunoz@gmail.com,alromero@mail.wvu.edu,smsingh@mix.wvu.edu,ukh0001@mix.wvu.edu,petavazohi@mix.wvu.edu,eric.bousquet@uliege.be,mailhexu@gmail.com",
    url="https://github.com/romerogroup/pyprocar",
    download_url="https://github.com/romerogroup/pyprocar/archive/5.5.1.tar.gz",
    packages=[
        "pyprocar",
        "pyprocar.utilsprocar",
        "pyprocar.procarparser",
        "pyprocar.procarfilefilter",
        "pyprocar.procarplot",
        "pyprocar.procarsymmetry",
        "pyprocar.procarunfold",
        "pyprocar.fermisurface",
        "pyprocar.fermisurface3d",
        "pyprocar.procarselect",
        "pyprocar.elkparser",
        "pyprocar.abinitparser",
        "pyprocar.doscarplot",
        "pyprocar.qeparser",
        "pyprocar.lobsterparser",
    ],
    license="LICENSE.txt",
    data_files=[("", ["LICENSE.txt"])],
    description="A Python library for electronic structure pre/post-processing.",
    install_requires=[
        "matplotlib",
        "seekpath",
        "scipy",
        "ase",
        "scikit-image",
        "pychemia",
        "pyvista",
        "trimesh",
    ],
    scripts=["scripts/procar.py"],
)



"""
Created on Fri May 10 16:23:30 2019

@author: Pedram Tavadze

"""

from .utilsprocar import UtilsProcar
from .procarparser import ProcarParser
from .procarselect import ProcarSelect
import numpy as np
from scipy.spatial import ConvexHull
from scipy.spatial import Voronoi
from skimage import measure
from multiprocessing import Pool
import scipy.interpolate as interpolate
from .splash import welcome


def get_wigner_seitz(recLat):
    kpoints = []
    for i in range(-1, 2):
        for j in range(-1, 2):
            for k in range(-1, 2):
                vec = i * recLat[0] + j * recLat[1] + k * recLat[2]
                kpoints.append(vec)
    brill = Voronoi(np.array(kpoints))
    faces = []
    for idict in brill.ridge_dict:
        if idict[0] == 13 or idict[1] == 13:
            faces.append(brill.ridge_dict[idict])
    verts = brill.vertices
    poly = []
    for ix in range(len(faces)):
        temp = []
        for iy in range(len(faces[ix])):
            temp.append(verts[faces[ix][iy]])
        poly.append(np.array(temp))
    return np.array(poly)


def mapping_func(kpoints, bands):
    kx = np.unique(kpoints[:, 0])
    ky = np.unique(kpoints[:, 1])
    kz = np.unique(kpoints[:, 2])
    mapped_func = np.zeros(shape=(len(kx), len(ky), len(kz)))
    kpoint_matrix = np.zeros(shape=(len(kx), len(ky), len(kz), 3))
    for ikx in range(len(kx)):
        cond1 = kpoints[:, 0] == kx[ikx]
        for iky in range(len(ky)):
            cond2 = kpoints[:, 1] == ky[iky]
            for ikz in range(len(kz)):
                cond3 = kpoints[:, 2] == kz[ikz]
                tot_cond = np.all([cond1, cond2, cond3], axis=0)
                if len(bands[tot_cond]) != 0:
                    mapped_func[ikx, iky, ikz] = bands[tot_cond][0]
                    kpoint_matrix[ikx, iky, ikz] = [kx[ikx], ky[iky], kz[ikz]]
                else:
                    mapped_func[ikx, iky, ikz] = np.nan
                    kpoint_matrix[ikx, iky, ikz] = [np.nan, np.nan, np.nan]

    return mapped_func, kpoint_matrix


# if the grid is gamma centered the gird would be something like [-0.45,-0.35,...,0,...,0.5]
# using FFT interpolate we will loose center. The fermi surface will not be symmetric with
# respect to the center. To avoid this we will add the points from 0.5 to -0.5 so the mesh will be like
# [-0.5,-0.45,..,0,...,0.45,0.5]
def symmetrize(data):
    # kpoints with one 0.5
    idx = (data.kpoints == 0.5).sum(axis=1) == 1
    #    idx = np.any(data.kpoints == 0.5,axis=1)
    kpoints_toAdd = data.kpoints[idx] + (data.kpoints[idx] == 0.5).astype(int) * -1
    bands_toAdd = data.bands[idx]
    spd_toAdd = data.spd[idx]
    # kpoints with two 0.5
    # -0.5,-0.5
    idx = (data.kpoints == 0.5).sum(axis=1) == 2
    kpoints_temp = data.kpoints[idx] + (data.kpoints[idx] == 0.5).astype(int) * -1
    bands_temp = data.bands[idx]
    spd_temp = data.spd[idx]

    kpoints_toAdd = np.append(kpoints_toAdd, kpoints_temp, axis=0)
    bands_toAdd = np.append(bands_toAdd, bands_temp, axis=0)
    spd_toAdd = np.append(spd_toAdd, spd_temp, axis=0)
    # [0.5,:,:]
    kpoints_temp = data.kpoints[idx][((data.kpoints[idx] == 0.5)[:, 0])] - [1, 0, 0]
    bands_temp = data.bands[idx][((data.kpoints[idx] == 0.5)[:, 0])]
    spd_temp = data.spd[idx][((data.kpoints[idx] == 0.5)[:, 0])]
    kpoints_toAdd = np.append(kpoints_toAdd, kpoints_temp, axis=0)
    bands_toAdd = np.append(bands_toAdd, bands_temp, axis=0)
    spd_toAdd = np.append(spd_toAdd, spd_temp, axis=0)
    # [:,0.5,:]
    kpoints_temp = data.kpoints[idx][((data.kpoints[idx] == 0.5)[:, 1])] - [0, 1, 0]
    bands_temp = data.bands[idx][((data.kpoints[idx] == 0.5)[:, 1])]
    spd_temp = data.spd[idx][((data.kpoints[idx] == 0.5)[:, 1])]
    kpoints_toAdd = np.append(kpoints_toAdd, kpoints_temp, axis=0)
    bands_toAdd = np.append(bands_toAdd, bands_temp, axis=0)
    spd_toAdd = np.append(spd_toAdd, spd_temp, axis=0)
    # [:,:,0.5]
    kpoints_temp = data.kpoints[idx][((data.kpoints[idx] == 0.5)[:, 2])] - [0, 0, 1]
    bands_temp = data.bands[idx][((data.kpoints[idx] == 0.5)[:, 2])]
    spd_temp = data.spd[idx][((data.kpoints[idx] == 0.5)[:, 2])]
    kpoints_toAdd = np.append(kpoints_toAdd, kpoints_temp, axis=0)
    bands_toAdd = np.append(bands_toAdd, bands_temp, axis=0)
    spd_toAdd = np.append(spd_toAdd, spd_temp, axis=0)
    # kpoints with three 0.5
    idx = (data.kpoints == 0.5).sum(axis=1) == 3
    kpoints_temp = np.array(
        [(x, y, z) for x in [-0.5, 0.5] for y in [-0.5, 0.5] for z in [-0.5, 0.5]]
    )[:-1]
    bands_temp = np.repeat(data.bands[idx], 7, axis=0)
    spd_temp = np.repeat(data.spd[idx], 7, axis=0)
    kpoints_toAdd = np.append(kpoints_toAdd, kpoints_temp, axis=0)
    bands_toAdd = np.append(bands_toAdd, bands_temp, axis=0)
    spd_toAdd = np.append(spd_toAdd, spd_temp, axis=0)

    data.kpoints = np.append(data.kpoints, kpoints_toAdd, axis=0)
    data.bands = np.append(data.bands, bands_toAdd, axis=0)
    data.spd = np.append(data.spd, spd_toAdd, axis=0)
    return data


def bring_pnts_to_BZ(recLat, kvector_cart, kvector_red, br_points):
    outsides = []
    directions = []
    # This section finds points that are outside of the 1st BZ and and creates those points in the 1st BZ
    movements = np.array(
        [
            recLat[0],
            recLat[1],
            recLat[2],
            -1 * recLat[0],
            -1 * recLat[1],
            -1 * recLat[2],
        ]
    )
    for ik in range(len(kvector_cart)):
        ik_copy = kvector_cart[ik][:]
        idirection = 0
        was_outside = False
        while is_outside([br_points, ik_copy]) and idirection < 6:
            ik_copy = kvector_cart[ik] + movements[idirection]
            idirection += 1
            was_outside = True
        if was_outside:
            outsides.append(kvector_red[ik])
            directions.append(
                np.dot(movements[idirection - 1], np.linalg.pinv(recLat)).round(2)
            )
        kvector_cart[ik] = ik_copy[:]
        kvector_red[ik] = np.dot(ik_copy, np.linalg.pinv(recLat)).round(2)
    if len(outsides):
        has_points_out = True
    else:
        has_points_out = False
    return kvector_cart, kvector_red, has_points_out


def fft_interpolate(function, scale):
    eigen_fft = np.fft.fftn(function)
    shifted_fft = np.fft.fftshift(eigen_fft)
    nx, ny, nz = np.array(shifted_fft.shape)
    pad_x = nx * (scale - 1) // 2
    pad_y = ny * (scale - 1) // 2
    pad_z = nz * (scale - 1) // 2
    new_matrix = np.pad(
        shifted_fft,
        ((pad_x, pad_x), (pad_y, pad_y), (pad_z, pad_z)),
        "constant",
        constant_values=0,
    )

    new_matrix = np.fft.ifftshift(new_matrix)
    interpolated = np.real(np.fft.ifftn(new_matrix)) * (scale ** 3)
    return interpolated


def is_outside(args):
    br_points = args[0]
    point = args[1]
    Hull1 = ConvexHull(br_points)
    added_points = np.append(br_points, [point], axis=0)
    Hull2 = ConvexHull(added_points)
    if len(Hull2.vertices) == len(Hull1.vertices):
        if sum(Hull2.vertices - Hull1.vertices) != 0:
            return True
    else:
        return True
    return False


def to_remove(args):
    faces = args[0]
    vert = args[1]
    return np.any(faces == vert, axis=1)


def fermi3D(procar, outcar, bands=-1, scale=1, mode="plain", st=False, **kwargs):
    """
    This function plots 3d fermi surface
    list of acceptable kwargs :
        plotting_package
        nprocess
        face_colors
        arrow_colors 
        arrow_spin
        atom
        orbital
        spin
        
    """
    welcome()

    # Initilizing the arguments :

    if "plotting_package" in kwargs:
        plotting_package = kwargs["plotting_package"]
    else:
        plotting_package = "mayavi"

    if "nprocess" in kwargs:
        nprocess = kwargs["nprocess"]
    else:
        nprocess = 2

    if "face_colors" in kwargs:
        face_colors = kwargs["face_colors"]
    else:
        face_colors = None
    if "cmap" in kwargs:
        cmap = kwargs["cmap"]
    else:
        cmap = "jet"
    if "atoms" in kwargs:
        atoms = kwargs["atoms"]
    else:
        atoms = [-1]  # project all atoms
    if "orbitals" in kwargs:
        orbitals = kwargs["orbitals"]
    else:
        orbitals = [-1]

    if "spin" in kwargs:
        spin = kwargs["spin"]
    else:
        spin = [0]
    if "mask_points" in kwargs:
        mask_points = kwargs["mask_points"]
    else:
        mask_points = 1
    if "energy" in kwargs:
        energy = kwargs["energy"]
    else:
        energy = 0
    if "transparent" in kwargs:
        transparent = kwargs["transparent"]
    else:
        transparent = False

    if "arrow_projection" in kwargs:
        arrow_projection = kwargs["arrow_projection"]
    else:
        arrow_projection = 2

    if plotting_package == "mayavi":
        try:
            from mayavi import mlab
            from tvtk.api import tvtk
        except:
            print(
                "You have selected mayavi as plottin package. please install mayavi and tvtk or choose a different package"
            )
            return
    elif plotting_package == "plotly":
        try:
            import plotly.plotly as py
            import plotly.figure_factory as ff
            import plotly.graph_objs as go
            
            cmap = mpl.cm.get_cmap(cmap)
            figs = []

        except:
            print(
                "You have selected plotly as plottin package. please install plotly or choose a different package"
            )
            return
    elif plotting_package == "matplotlib":
        try:
            import matplotlib.pylab as plt
            from mpl_toolkits.mplot3d.art3d import Poly3DCollection
        except:
            print(
                "You have selected matplotlib as plotting package. please install matplotlib or choose a different package"
            )
            return
    elif plotting_package == "ipyvolume":
        try:
            import ipyvolume.pylab as ipv
        except:
            print(
                "You have selected ipyvolume as plotting package. please install ipyvolume or choose a different package"
            )
            return
    if mode == 'colorful' :
        face_colors = [(1, 0, 0),
                       (0, 1, 0),
                       (0, 0, 1),
                       (1, 1, 0),
                       (0, 1, 1),
                       (1, 0, 1),
                       (192/255, 192/255, 192/255),
                       (128/255, 128/255, 128/255),
                       (128/255, 0, 0),
                       (128/255, 128/255, 0),
                       (0, 128/255, 0),
                       (128/255, 0, 128/255),
                       (0, 128/255, 128/255),
                       (0, 0, 128/255)]

    permissive = False

    # get fermi from outcar
    outcarparser = UtilsProcar()
    e_fermi = outcarparser.FermiOutcar(outcar)
    print("Fermi=", e_fermi)
    e_fermi += energy
    # get reciprocal lattice from outcar
    recLat = outcarparser.RecLatOutcar(outcar)

    # parsing the Procar file
    procarFile = ProcarParser()
    procarFile.readFile(procar, permissive)

    poly = get_wigner_seitz(recLat)
    # plot brilliouin zone
    if plotting_package == "mayavi":
        brillouin_point = []
        brillouin_faces = []
        point_count = 0
        for iface in poly:
            single_face = []
            for ipoint in iface:
                single_face.append(point_count)
                brillouin_point.append(list(ipoint))
                point_count += 1
            brillouin_faces.append(single_face)
        polydata_br = tvtk.PolyData(points=brillouin_point, polys=brillouin_faces)
        mlab.figure(figure=None, bgcolor=(1,1,1), fgcolor=None, engine=None, size=(400, 350))
        mlab.pipeline.surface(
            polydata_br,
            representation="wireframe",
            color=(0, 0, 0),
            line_width=4,
            name="BRZ",
        )
    elif plotting_package == "plotly":

        for iface in poly:
            iface = np.pad(iface, ((0, 1), (0, 0)), "wrap")
            x, y, z = iface[:, 0], iface[:, 1], iface[:, 2]
            plane = go.Scatter3d(
                x=x, y=y, z=z, mode="lines", line=dict(color="black", width=4)
            )
            figs.append(plane)

    elif plotting_package == "matplotlib":
        fig = plt.figure()
        ax = fig.add_subplot(111, projection="3d")
        brillouin_zone = Poly3DCollection(
            poly, facecolors=["None"] * len(poly), alpha=1, linewidth=4
        )
        brillouin_zone.set_edgecolor("k")
        ax.add_collection3d(brillouin_zone, zs=0, zdir="z")

    br_points = []
    for iface in poly:
        for ipoint in iface:
            br_points.append(ipoint)
    br_points = np.unique(br_points, axis=0)
    print("Number of bands: %d" % procarFile.bandsCount)
    print("Number of koints %d" % procarFile.kpointsCount)
    print("Number of ions: %d" % procarFile.ionsCount)
    print("Number of orbitals: %d" % procarFile.orbitalCount)
    print("Number of spins: %d" % procarFile.ispin)

    # selecting the data
    data = ProcarSelect(procarFile, deepCopy=True)
    if bands == -1:
        bands = range(data.bands.shape[1])

    kvector = data.kpoints
    kmax = np.max(kvector)
    kmin = np.min(kvector)

    if abs(kmax) != abs(kmin):
        print("The mesh provided is gamma center, symmetrizing data")
        print("For a better fermi surface, use a non-gamma centered k-mesh")
        data = symmetrize(data)
        kvector = data.kpoints

    kvector_red = data.kpoints.copy()
    kvector_cart = np.dot(kvector_red, recLat)

    # This section finds points that are outside of the 1st BZ and and creates those points in the 1st BZ
    kvector_cart, kvector_red, has_points_out = bring_pnts_to_BZ(
        recLat, kvector_cart, kvector_red, br_points
    )
    #    has_points_out = False

    # getting the mesh grid in each dirrection
    kx_red = np.unique(kvector_red[:, 0])
    ky_red = np.unique(kvector_red[:, 1])
    kz_red = np.unique(kvector_red[:, 2])

    # getting the lengths between kpoints in each direction
    klength_x = np.abs(kx_red[-1] - kx_red[-2])
    klength_y = np.abs(ky_red[-1] - ky_red[-2])
    klength_z = np.abs(kz_red[-1] - kz_red[-2])
    klengths = [klength_x, klength_y, klength_z]

    # getting number of kpoints in each direction with the addition of kpoints needed to sample the 1st BZ fully (in reduced)
    nkx_red = kx_red.shape[0]
    nky_red = ky_red.shape[0]
    nkz_red = kz_red.shape[0]

    # getting numner of kpoints in each direction provided by vasp
    nkx_orig = np.unique(kvector[:, 0]).shape[0]
    nky_orig = np.unique(kvector[:, 1]).shape[0]
    nkz_orig = np.unique(kvector[:, 2]).shape[0]

    # Amount of kpoints needed to add on to fully sample 1st BZ
    padding_x = (nkx_red - nkx_orig) // 2
    padding_y = (nky_red - nky_orig) // 2
    padding_z = (nkz_red - nkz_orig) // 2

    if mode == "parametric":
        data.selectIspin(spin)
        data.selectAtoms(atoms, fortran=False)
        data.selectOrbital(orbitals)
    elif mode == "external":
        if "color_file" in kwargs:
            rf = open(kwargs["color_file"])

            lines = rf.readlines()
            counter = 0
            color_kvector = []
            color_eigen = []
            for iline in lines:
                if counter < 2:
                    if "band" in iline:
                        counter += 1
                        continue
                    temp = [float(x) for x in iline.split()]
                    color_kvector.append([temp[0], temp[1], temp[2]])
            counter = -1
            for iline in lines:
                if "band" in iline:
                    counter += 1
                    iband = int(iline.split()[-1])
                    color_eigen.append([])
                    continue
                color_eigen[counter].append(float(iline.split()[-1]))
            rf.close()

            color_kvector = np.array(color_kvector)
            color_kvector_red = color_kvector.copy()
            color_kvector_cart = np.dot(color_kvector, recLat)
            if has_points_out:

                color_kvector_cart, color_kvector_red, temp = bring_pnts_to_BZ(
                    recLat, color_kvector_cart, color_kvector_red, br_points
                )
        else:
            print("mode selected was external, but no color_file name was provided")
            return
    if st:

        dataX = ProcarSelect(procarFile, deepCopy=True)
        dataY = ProcarSelect(procarFile, deepCopy=True)
        dataZ = ProcarSelect(procarFile, deepCopy=True)

        dataX.kpoints = data.kpoints
        dataY.kpoints = data.kpoints
        dataZ.kpoints = data.kpoints

        dataX.spd = data.spd
        dataY.spd = data.spd
        dataZ.spd = data.spd

        dataX.bands = data.bands
        dataY.bands = data.bands
        dataZ.bands = data.bands

        dataX.selectIspin([1])
        dataY.selectIspin([2])
        dataZ.selectIspin([3])

        dataX.selectAtoms(atoms, fortran=False)
        dataY.selectAtoms(atoms, fortran=False)
        dataZ.selectAtoms(atoms, fortran=False)

        dataX.selectOrbital(orbitals)
        dataY.selectOrbital(orbitals)
        dataZ.selectOrbital(orbitals)
    ic = 0
    for iband in bands:

        print("Plotting band %d" % iband)

        eigen = data.bands[:, iband]

        # mapping the eigen values on the mesh grid to a matrix
        mapped_func, kpoint_matrix = mapping_func(kvector, eigen)

        # adding the points from the 2nd BZ to 1st BZ to fully sample the BZ. Check np.pad("wrap") for more information
        mapped_func = np.pad(
            mapped_func,
            ((padding_x, padding_x), (padding_y, padding_y), (padding_z, padding_z)),
            "wrap",
        )

        # Fourier interpolate the mapped function E(x,y,z)
        surf_equation = fft_interpolate(mapped_func, scale)

        # after the FFT we loose the center of the BZ, using numpy roll we bring back the center of the BZ
        surf_equation = np.roll(surf_equation, (scale) // 2, axis=[0, 1, 2])

        try:
            # creating the isosurface if possible
            verts, faces, normals, values = measure.marching_cubes_lewiner(
                surf_equation, e_fermi
            )

        except:

            print("No isosurface for this band")
            continue
        # the vertices provided are scaled and shifted to start from zero
        # we center them to zero, and rescale them to fit the real BZ by multiplying by the klength in each direction
        for ix in range(3):
            verts[:, ix] -= verts[:, ix].min()
            verts[:, ix] -= (verts[:, ix].max() - verts[:, ix].min()) / 2
            verts[:, ix] *= klengths[ix] / scale

        # the vertices need to be transformed to reciprocal spcae from recuded space, to find the points that are
        # in 2nd BZ, to be removed
        verts = np.dot(verts, recLat)

        # identifying the points in 2nd BZ and removing them
        if has_points_out:
            args = []
            for ivert in range(len(verts)):
                args.append([br_points, verts[ivert]])

            p = Pool(nprocess)
            results = np.array(p.map(is_outside, args))
            p.close()
            out_verts = np.arange(0, len(results))[results]
            new_faces = []
            #            outs_bool_mat = np.zeros(shape=faces.shape,dtype=np.bool)

            for iface in faces:
                remove = False
                for ivert in iface:
                    if ivert in out_verts:
                        remove = True

                        continue

                if not remove:
                    new_faces.append(iface)
            faces = np.array(new_faces)

        print("done removing")
        # At this point we have the plain Fermi surface, we can color the surface depending on the projection
        # We create the center of faces by averaging coordinates of corners

        if mode == "parametric":

            character = data.spd[:, iband]

            centers = np.zeros(shape=(len(faces), 3))
            for iface in range(len(faces)):
                centers[iface, 0:3] = np.average(verts[faces[iface]], axis=0)

            colors = interpolate.griddata(
                kvector_cart, character, centers, method="nearest"
            )
        elif mode == "external":
            character = np.array(color_eigen[ic])
            ic += 1
            centers = np.zeros(shape=(len(faces), 3))
            for iface in range(len(faces)):
                centers[iface, 0:3] = np.average(verts[faces[iface]], axis=0)

            colors = interpolate.griddata(
                color_kvector_cart, character, centers, method="nearest"
            )

        if st:
            projection_x = dataX.spd[:, iband]
            projection_y = dataY.spd[:, iband]
            projection_z = dataZ.spd[:, iband]

            verts_spin, faces_spin, normals, values = measure.marching_cubes_lewiner(
                mapped_func, e_fermi
            )

            for ix in range(3):
                verts_spin[:, ix] -= verts_spin[:, ix].min()
                verts_spin[:, ix] -= (
                    verts_spin[:, ix].max() - verts_spin[:, ix].min()
                ) / 2
                verts_spin[:, ix] *= klengths[ix]
            verts_spin = np.dot(verts_spin, recLat)

            if has_points_out:
                args = []
                for ivert in range(len(verts_spin)):
                    args.append([br_points, verts_spin[ivert]])

                p = Pool(nprocess)
                results = np.array(p.map(is_outside, args))
                p.close()
                out_verts = np.arange(0, len(results))[results]

                new_faces = []
                for iface in faces_spin:
                    remove = False
                    for ivert in iface:
                        if ivert in out_verts:
                            remove = True
                            continue
                    if not remove:
                        new_faces.append(iface)
                faces_spin = np.array(new_faces)

            centers = np.zeros(shape=(len(faces_spin), 3))
            for iface in range(len(faces_spin)):
                centers[iface, 0:3] = np.average(verts_spin[faces_spin[iface]], axis=0)

            colors1 = interpolate.griddata(
                kvector_cart, projection_x, centers, method="linear"
            )
            colors2 = interpolate.griddata(
                kvector_cart, projection_y, centers, method="linear"
            )
            colors3 = interpolate.griddata(
                kvector_cart, projection_z, centers, method="linear"
            )
            spin_arrows = np.vstack((colors1, colors2, colors3)).T

        if plotting_package == "mayavi":
            polydata = tvtk.PolyData(points=verts, polys=faces)

            if face_colors != None:
                mlab.pipeline.surface(
                    polydata,
                    representation="surface",
                    color=face_colors[ic],
                    opacity=1,
                    name="band-" + str(iband),
                )
                ic += 1
            else:
                if mode == "plain":
                    if not (transparent):
                        s = mlab.pipeline.surface(
                            polydata,
                            representation="surface",
                            color=(0, 0.5, 1),
                            opacity=1,
                            name="band-" + str(iband),
                        )


                elif mode == "parametric" or mode == "external":

                    polydata.cell_data.scalars = colors
                    polydata.cell_data.scalars.name = "celldata"
                    mlab.pipeline.surface(
                        polydata, vmin=0, vmax=colors.max(), colormap=cmap
                    )
                    cb = mlab.colorbar(orientation="vertical")

            if st:
                x, y, z = list(zip(*centers))
                u, v, w = list(zip(*spin_arrows))

                pnts = mlab.quiver3d(
                    x,
                    y,
                    z,
                    u,
                    v,
                    w,
                    line_width=5,
                    mode="arrow",
                    resolution=25,
                    reset_zoom=False,
                    name="spin-" + str(iband),
                    mask_points=mask_points,
                    scalars=spin_arrows[:, arrow_projection],
                    vmin=-1,
                    vmax=1,
                    colormap=cmap,
                )
                pnts.glyph.color_mode = "color_by_scalar"
                pnts.glyph.glyph_source.glyph_source.shaft_radius = 0.05
                pnts.glyph.glyph_source.glyph_source.tip_radius = 0.1

        elif plotting_package == "plotly":
            if mode == "plain":
                if not (transparent):
                    x, y, z = zip(*verts)
                    fig = ff.create_trisurf(
                        x=x,
                        y=y,
                        z=z,
                        plot_edges=False,
                        simplices=faces,
                        title="band-%d" % ic,
                    )

                    figs.append(fig["data"][0])

            elif mode == "parametric" or mode == "external":

                face_colors = cmap(colors)
                colormap = [
                    "rgb(%i,%i,%i)" % (x[0], x[1], x[2])
                    for x in (face_colors * 255).round()
                ]
                x, y, z = zip(*verts)
                fig = ff.create_trisurf(
                    x=x,
                    y=y,
                    z=z,
                    plot_edges=False,
                    colormap=colormap,
                    simplices=faces,
                    show_colorbar=True,
                    title="band-%d" % ic,
                )

                figs.append(fig["data"][0])
        elif plotting_package == "matplotlib":
            if mode == "plain":
                x, y, z = zip(*verts)
                ax.plot_trisurf(x, y, faces, z, linewidth=0.2, antialiased=True)
            elif mode == "parametric" or mode == "external":
                print(
                    "coloring the faces is not implemented in matplot lib, please use another plotting package.we recomend mayavi."
                )
        elif plotting_package == "ipyvolume":
            if mode == "plain":
                ipv.figure()
                ipv.plot_trisurf(verts[:, 0], verts[:, 1], verts[:, 2], triangles=faces)

            elif mode == "paramteric" or mode == "external":
                face_colors = cmap(colors)
                colormap = [
                    "rgb(%i,%i,%i)" % (x[0], x[1], x[2])
                    for x in (face_colors * 255).round()
                ]
                ipv.figure()
                ipv.plot_trisurf(
                    verts[:, 0], verts[:, 1], verts[:, 2], triangles=faces, color=cmap
                )

    # if plotting_package == "mayavi":
    #     mlab.colorbar(orientation="vertical") 
    #     mlab.show()
    # elif plotting_package == "plotly":
    #     layout = go.Layout(showlegend=False)

    #     fig = go.Figure(data=figs, layout=layout)
    #     py.iplot(fig)
    # elif plotting_package == "matplotlib":
    #     plt.show()
    # elif plotting_package == "ipyvolume":
    #     ipv.show()

    return



"""
Created on May 17 2020 
@author: Pedram Tavadze
"""
import matplotlib as mpl
import matplotlib.pylab as plt
import numpy as np

np.seterr(divide="ignore", invalid="ignore")
figsize = (12, 6)

class DosPlot:
    def __init__(self, vaspxml="vasprun.xml"):
        from pychemia.code.vasp import VaspXML

        self.VaspXML = VaspXML(vaspxml)
        return
    
    def plot_total(
            self,
            spins=None,
            spin_colors=None,
            figsize=figsize,
            ax=None,
            orientation="horizontal",
            labels=None,
    ):

        # dos is a pychemia density of states object
        # pychemia.visual.DensityOfStates
        if spin_colors is None:
            spin_colors = [(1, 0, 0), (0, 0, 1)]
        dos = self.VaspXML.dos_total
        
        fig, ax = plotter(
            dos,
            spins,
            spin_colors,
            figsize,
            ax,
            orientation,
            labels,
        )

        return fig, ax

    def plot_parametric_line(
        self,
        atoms=None,
        spins=None,
        orbitals=None,
        spin_colors=None,
        figsize=(12, 6),
        ax=None,
        orientation="horizontal",
        labels=None,
    ):
        if ax is None:
            if orientation == "horizontal":
                fig = plt.figure(figsize=figsize)
                ax = fig.add_subplot(111)
            elif orientation == "vertical":
                fig = plt.figure(figsize=(figsize[1], figsize[0]))
                ax = fig.add_subplot(111)
        else:
            fig = ax.get_figure()
        if spin_colors is None:
            spin_colors = [(1, 0, 0), (0, 0, 1)]
        dos = self.VaspXML.dos_parametric(atoms=atoms, spin=spins, orbitals=orbitals)

        fig, ax = plotter(
            dos,
            spins,
            spin_colors,
            figsize,
            ax,
            orientation,
            labels,
        )
        return fig, ax

    def plot_parametric(
        self,
        atoms=None,
        spins=None,
        orbitals=None,
        spin_colors=None,
        cmap="jet",
        vmin=0,
        vmax=1,
        elimit=None,
        figsize=(12, 6),
        ax=None,
        orientation="horizontal",
        labels=None,
        plot_bar=True,
    ):
        if ax is None:
            if orientation == "horizontal":
                fig = plt.figure(figsize=figsize)
                ax = fig.add_subplot(111)
            elif orientation == "vertical":
                fig = plt.figure(figsize=(figsize[1], figsize[0]))
                ax = fig.add_subplot(111)
        else:
            fig = ax.get_figure()
        if spin_colors is None:
            spin_colors = [(1, 0, 0), (0, 0, 1)]
        cmap = mpl.cm.get_cmap(cmap)
        dos_total = self.VaspXML.dos_total
        dos_total_projected = self.VaspXML.dos_parametric()
        dos = self.VaspXML.dos_parametric(atoms=atoms, spin=spins, orbitals=orbitals)

        if ax is None:
            if orientation == "horizontal":
                fig = plt.figure(figsize=figsize)
                ax = fig.add_subplot(111)
            elif orientation == "vertical":
                fig = plt.figure(figsize=(figsize[1], figsize[0]))
                ax = fig.add_subplot(111)
        else:
            fig = ax.get_figure()
        if plot_bar:
            norm = mpl.colors.Normalize(vmin=vmin, vmax=vmax)
            fig.colorbar(mpl.cm.ScalarMappable(norm=norm, cmap=cmap), ax=ax)
        if not elimit:
            elimit = [dos.energies[0], dos.energies[1]]
        cond1 = dos.energies > elimit[0]
        cond2 = dos.energies < elimit[1]
        cond = np.all([cond1, cond2], axis=0)

        dE = dos.energies[1] - dos.energies[0]
        for ispin in spins:
            x = []
            y_total = []
            bar_color = []
            for idos in range(len(dos.energies[cond])):
                x.append(dos.energies[cond][idos])
                y = dos.dos[cond, ispin + 1][idos]
                y_total.append(dos_total.dos[cond, ispin + 1][idos])
                y_total_projected = dos_total_projected.dos[cond, ispin + 1][idos]
                if ispin > 0 and len(spins) > 1:
                    y *= -1
                    y_total[-1] *= -1
                    y_total_projected *= -1
                bar_color.append(cmap(y / (y_total_projected * (vmax - vmin))))
            if orientation == "horizontal":
                #                ax.bar(x,y_total,dE,color=bar_color)
                for idos in range(len(x) - 1):
                    ax.fill_between(
                        [x[idos], x[idos + 1]],
                        [y_total[idos], y_total[idos + 1]],
                        color=bar_color[idos],
                    )
            elif orientation == "vertical":
                for idos in range(len(x) - 1):
                    ax.fill_betweenx(
                        [x[idos], x[idos + 1]],
                        [y_total[idos], y_total[idos + 1]],
                        color=bar_color[idos],
                    )
        #                ax.barh(y_total,x,dE,color=bar_color)

        return fig, ax

    def plot_stack_species(
        self,
        spins=None,
        orbitals=None,
        spin_colors=None,
        colors=None,
        elimit=None,
        figsize=(12, 6),
        ax=None,
        orientation="horizontal",
    ):
        if spin_colors is None:
            spin_colors = [(0, 0, 1), (1, 0, 0)]
        if colors is None:
            colors=['red',
                    'green',
                    'blue',
                    'cyan',
                    'magenta',
                    'yellow',
                    'orange',
                    'purple',
                    'brown',
                    'navy',
                    'maroon',
                    'olive']

        #        if ax is None:
        #            if orientation == 'horizontal':
        #                fig = plt.figure(figsize=figsize)
        #                ax = fig.add_subplot(111)
        #            elif orientation == 'vertical':
        #                fig = plt.figure(figsize=(figsize[1], figsize[0]))
        #                ax = fig.add_subplot(111)
        if ax is None:
            if orientation == "horizontal":
                fig = plt.figure(figsize=figsize)
                ax = fig.add_subplot(111)
            elif orientation == "vertical":
                fig = plt.figure(figsize=(figsize[1], figsize[0]))
                ax = fig.add_subplot(111)
        else:
            fig = ax.get_figure()

        if not elimit:
            elimit = [-2, 2]
        dos_projected_total = self.VaspXML.dos_parametric(spin=spins, orbitals=orbitals)
        if self.VaspXML.dos_projected[0].ncols == (1 + 3 + 5) * 2:
            all_orbitals = "spd"
        elif self.VaspXML.dos_projected[0].ncols == (1 + 3 + 5 + 7) + 1:
            all_orbitals = "spdf"
        else:
            all_orbitals = ""
        label = ""
        if orbitals:
            print("The plot only considers orbitals", orbitals)
            label = "-"
            if sum([x in orbitals for x in [0]]) == 1:
                label += "s"
            if sum([x in orbitals for x in [1, 2, 3]]) == 3:
                label += "p"
            if sum([x in orbitals for x in [4, 5, 6, 7, 8]]) == 5:
                label += "d"
            if sum([x in orbitals for x in [9, 10, 11, 12, 13, 14, 15]]) == 7:
                label += "f"
            if label == "-" + all_orbitals:
                label = ""
        dos_total = self.VaspXML.dos_total

        cond1 = self.VaspXML.dos_total.energies > elimit[0]
        cond2 = self.VaspXML.dos_total.energies < elimit[1]
        cond = np.all([cond1, cond2], axis=0)

        for ispin in spins:
            bottom = np.zeros_like(self.VaspXML.dos_total.energies[cond])
            for ispc in range(len(self.VaspXML.species)):
                idx = (
                    np.array(self.VaspXML.initial_structure.symbols)
                    == self.VaspXML.species[ispc]
                )
                atoms = list(np.where(idx)[0])
                dos = self.VaspXML.dos_parametric(
                    atoms=atoms, spin=spins, orbitals=orbitals
                )

                x = dos.energies[cond]
                y = (
                    dos.dos[cond, ispin + 1] * dos_total.dos[cond, ispin + 1]
                ) / dos_projected_total.dos[cond, ispin + 1]

                if ispin > 0 and len(spins) > 1:
                    y *= -1
                    if orientation == "horizontal":
                        ax.fill_between(x, bottom + y, bottom, color=colors[ispc])
                    elif orientation == "vertical":
                        ax.fill_betweenx(x, bottom + y, bottom, color=colors[ispc])
                else:
                    if orientation == "horizontal":
                        ax.fill_between(
                            x,
                            bottom + y,
                            bottom,
                            color=colors[ispc],
                            label=self.VaspXML.species[ispc] + label,
                        )
                    elif orientation == "vertical":
                        ax.fill_betweenx(
                            x,
                            bottom + y,
                            bottom,
                            color=colors[ispc],
                            label=self.VaspXML.species[ispc] + label,
                        )
                bottom += y
        return fig, ax

    def plot_stack_orbitals(
        self,
        spins=None,
        atoms=None,
        spin_colors=None,
        colors=None,
        elimit=None,
        figsize=(12, 6),
        ax=None,
        orientation="horizontal",
    ):
        if spin_colors is None:
            spin_colors = [(0, 0, 0), (0, 0, 0)]
        if colors is None:
            colors=['red',
                    'blue',
                    'green',
                    'cyan',
                    'magenta',
                    'yellow',
                    'orange',
                    'purple',
                    'brown',
                    'navy',
                    'maroon',
                    'olive']

        if ax is None:
            if orientation == "horizontal":
                fig = plt.figure(figsize=figsize)
                ax = fig.add_subplot(111)
            elif orientation == "vertical":
                fig = plt.figure(figsize=(figsize[1], figsize[0]))
                ax = fig.add_subplot(111)
        else:
            fig = ax.get_figure()
        atom_names = ""
        if atoms:
            print(
                "The plot only considers atoms",
                np.array(self.VaspXML.initial_structure.symbols)[atoms],
            )
            atom_names = ""
            for ispc in np.unique(
                np.array(self.VaspXML.initial_structure.symbols)[atoms]
            ):
                atom_names += ispc + "-"
        all_atoms = ""
        for ispc in np.unique(np.array(self.VaspXML.initial_structure.symbols)):
            all_atoms += ispc + "-"
        if atom_names == all_atoms:
            atom_names = ""
        dos_total = self.VaspXML.dos_total
        dos_projected_total = self.VaspXML.dos_parametric()

        if not elimit:
            elimit = [-2, 2]
        dos_projected_total = self.VaspXML.dos_parametric()

        cond1 = self.VaspXML.dos_total.energies > elimit[0]
        cond2 = self.VaspXML.dos_total.energies < elimit[1]
        cond = np.all([cond1, cond2], axis=0)
        orb_names = ["s", "p", "d"]
        orb_l = [[0], [1, 2, 3], [4, 5, 6, 7, 8]]
        for ispin in spins:
            bottom = np.zeros_like(self.VaspXML.dos_total.energies[cond])
            for iorb in range(3):
                dos = self.VaspXML.dos_parametric(
                    atoms=atoms, spin=spins, orbitals=orb_l[iorb]
                )
                x = dos.energies[cond]
                y = (
                    dos.dos[cond, ispin + 1] * dos_total.dos[cond, ispin + 1]
                ) / dos_projected_total.dos[cond, ispin + 1]
                y = np.nan_to_num(y, 0)

                if ispin > 0 and len(spins) > 1:
                    y *= -1
                    if orientation == "horizontal":
                        ax.fill_between(x, bottom + y, bottom, color=colors[iorb])
                    elif orientation == "vertical":
                        ax.fill_betweenx(x, bottom + y, bottom, color=colors[iorb])
                else:
                    if orientation == "horizontal":
                        ax.fill_between(
                            x,
                            bottom + y,
                            bottom,
                            color=colors[iorb],
                            label=atom_names + orb_names[iorb],
                        )
                    elif orientation == "vertical":
                        ax.fill_betweenx(
                            x,
                            bottom + y,
                            bottom,
                            color=colors[iorb],
                            label=atom_names + orb_names[iorb],
                        )
                bottom += y
        return fig, ax

    def plot_stack(
        self,
        items={},
        spins=None,
        spin_colors=None,
        colors=None,
        elimit=None,
        figsize=(12, 6),
        ax=None,
        orientation="horizontal",
    ):

        if len(items) == 0:
            print(
                """Please provide the stacking items in which you want to plot,
                  example : {'Sr':[1,2,3],'O':[4,5,6,7,8]} will plot the stacked
                  plots of p orbitals of Sr and d orbitals of Oxygen."""
            )

        if spin_colors is None:
            spin_colors = [(0, 0, 1), (1, 0, 0)]
        src_colors = colors
        if src_colors is None:
            src_colors=['red',
                        'blue',
                        'green',
                        'cyan',
                        'magenta',
                        'yellow',
                        'orange',
                        'purple',
                        'brown',
                        'navy',
                        'maroon',
                        'olive']

        if ax is None:
            if orientation == "horizontal":
                fig = plt.figure(figsize=figsize)
                ax = fig.add_subplot(111)
            elif orientation == "vertical":
                fig = plt.figure(figsize=(figsize[1], figsize[0]))
                ax = fig.add_subplot(111)
        else:
            fig = ax.get_figure()
        if not elimit:
            elimit = [-2, 2]
        dos_total = self.VaspXML.dos_total

        if self.VaspXML.dos_projected[0].ncols == (1 + 3 + 5) * dos_total.ncols:
            all_orbitals = "spd"
        elif self.VaspXML.dos_projected[0].ncols == (1 + 3 + 5 + 7) * dos_total.ncols:
            all_orbitals = "spdf"
        else:
            all_orbitals = ""

        cond1 = self.VaspXML.dos_total.energies > elimit[0]
        cond2 = self.VaspXML.dos_total.energies < elimit[1]
        cond = np.all([cond1, cond2], axis=0)
        counter = 0
        colors = {}

        for ispc in items:
            colors[ispc] = src_colors[counter]
            counter += 1

        dos_projected_total = self.VaspXML.dos_parametric(spin=spins)

        for ispin in spins:
            bottom = np.zeros_like(self.VaspXML.dos_total.energies[cond])
            for ispc in items:
                idx = np.array(self.VaspXML.initial_structure.symbols) == ispc
                atoms = list(np.where(idx)[0])
                orbitals = items[ispc]

                dos = self.VaspXML.dos_parametric(
                    atoms=atoms, spin=spins, orbitals=orbitals
                )

                label = "-"
                if sum([x in orbitals for x in [0]]) == 1:
                    label += "s"
                if sum([x in orbitals for x in [1, 2, 3]]) == 3:
                    label += "p"
                if sum([x in orbitals for x in [4, 5, 6, 7, 8]]) == 5:
                    label += "d"
                if sum([x in orbitals for x in [9, 10, 11, 12, 13, 14, 15]]) == 7:
                    label += "f"
                if label == "-" + all_orbitals:
                    label = ""
                x = dos.energies[cond]

                y = (
                    dos.dos[cond, ispin + 1] * dos_total.dos[cond, ispin + 1]
                ) / dos_projected_total.dos[cond, ispin + 1]
                if ispin > 0 and len(spins) > 1:
                    y *= -1
                    if orientation == "horizontal":
                        ax.fill_between(x, bottom + y, bottom, color=colors[ispc])
                    elif orientation == "vertical":
                        ax.fill_betweenx(x, bottom + y, bottom, color=colors[ispc])
                else:
                    if orientation == "horizontal":
                        ax.fill_between(
                            x,
                            bottom + y,
                            bottom,
                            color=colors[ispc],
                            label=ispc + label,
                        )
                    elif orientation == "vertical":

                        ax.fill_betweenx(
                            x,
                            bottom + y,
                            bottom,
                            color=colors[ispc],
                            label=ispc + label,
                        )
                bottom += y
        return fig, ax


def plotter(dos,
            spins,
            spin_colors,
            figsize,
            ax,
            orientation,
            labels
):

    if orientation == "horizontal":
        if ax is None:
            fig = plt.figure(figsize=figsize)
            ax = fig.add_subplot(111)
        else:
            fig = ax.get_figure()

        x = dos.energies

        for iy in spins:
            y = dos.dos[:, iy + 1]
            if iy > 0 and len(spins) > 1:
                y *= -1
            if not labels is None:
                ax.plot(
                    x,
                    y,
                    "r-",
                    color=spin_colors[iy],
                    label=labels[iy],
                )
            else:
                ax.plot(
                    x,
                    y,
                    "r-",
                    color=spin_colors[iy],
                )

    elif orientation == "vertical":
        if ax is None:
            fig = plt.figure(figsize=(figsize[1], figsize[0]))
            ax = fig.add_subplot(111)
        else:
            fig = plt.gca()
        y = dos.energies
        for ix in spins:
            x = dos.dos[:, ix + 1]
            if ix > 0 and len(spins) > 1:
                x *= -1
            if not labels is None:
                ax.plot(
                    x,
                    y,
                    "r-",
                    color=spin_colors[ix],
                    label=labels[ix],
                )
            else:
                ax.plot(
                    x,
                    y,
                    "r-",
                    color=spin_colors[ix],
                )
    return fig, ax
